#!/usr/bin/env python
"""
This class generate the full information showed in the history page of T0Mon
"""
from WMCore.Database.DBFormatter import DBFormatter
from math import ceil

class AllHistory(DBFormatter):
    """  
    _This class collect information from different tables in order to have a complete view of the data connected to a single
   (Streamer | (Express | Reco | Repack | AlcaSkim) +[Merge] + (file | job)). These are also all the possible input values_
    """
    #Ids= save the Ids for the * files tables. This values are used to connect the different sets of tables
    Ids ={'strm': [],'exprF': [], 'exprJ':[],'exprMJ': [], 'exprMF':[],'repkJ':[],'repkF':[],'repkMJ':[],'repkMF':[], 'recoJ':[],'recoF':[],'recoMJ':[],'recoMF':[],'alcaJ':[],'alcaF':[],'alcaMJ':[],'alcaMF':[],'StraightToMerge':[]}
    tableNames={'strm':"StreamerFiles",'exprF':"ExpressFiles",'exprJ':"ExpressJobs",'exprMJ':"ExpressMergeJobs",'exprMF':"ExpressMergeFiles",'repkF':"RepackFiles",'repkJ':"RepackJobs",'repkMJ':"RepackMergeJobs",'repkMF':"RepackMergeFiles",'recoF':"RecoFiles",'recoJ':"RecoJobs",'recoMJ':"RecoMergeJobs",'recoMF':"RecoMergeFiles",'alcaF':"AlcaFiles",'alcaJ':"AlcaJobs",'alcaMJ':"AlcaMergeJobs",'alcaMF':"AlcaMergeFiles"}
  #
    orderExpr={'strm': ['exprJ','exprF'],'exprJ':['strm'],'exprF':['exprMF','exprMJ','strm'],'exprMF':['exprMJ'],'exprMJ':['exprF']} 
    
    orderRepk={'strm': ['repkJ','repkF'],'repkJ':['strm'],'repkF':['repkMF','repkMJ','repkJ'],'repkMF':['repkMJ','recoJ','recoF','repkJ'],'repkMJ':['repkF'], 'recoF':['recoMF','recoMJ','recoJ','alcaF','alcaJ'],'recoJ':['recoF','repkMF'],'recoMJ':['recoF'],'recoMF':['recoF'],'alcaF':['alcaMF','alcaMJ','alcaJ'],'alcaJ':['recoF'], 'alcaMF':['alcaF','alcaMJ'], 'alcaMJ':['alcaF']}
    MAXDIM=200
    checkedStraightToMerge=False
    # Output table names
    
  
    
    def execute(self,vtype,value, conn = None, transaction = False):
        """
        analysis of the parameters and running: generateAllIds() and then buildTables()
        """
        vtype=vtype.lower()
        self.conn = conn
        self.transaction = transaction
        self.vtype=vtype
        self.value=value
        
        self.order=self.orderRepk
        if self.vtype[0:4]=="expr" or self.strmExprGreaterThenBulks() :
            self.order=self.orderExpr
        current=None
        for key in self.Ids:
          self.Ids[key]=[]
       
        try: 
          if len(vtype)>4:
           current = vtype[0:4]+vtype[4:].upper()
          else: current = vtype
          self.Ids[current]=[value,] 
          
        except Exception, e:
           print e
           current=None
        
        if current not in self.Ids.keys():
          return {"Error" : "wrong input type",'current': vtype,'allowed vtype values: ': "strm|(repk|reco|alca|expr)+[M]+(j|f)"} 
        self.generateAllIds(current)
        #for i in self.Ids:
            #print "Ids[%s]=%s"%(i,self.Ids[i])
        return self.buildTables()
          
        
    def generateAllIds(self,current):
        """
        This method scan the tree from the starting point (current) recursively and fill the Ids dictionary with all the different table ids.
        """
        if len(self.Ids[current])>0:
          if (current in self.order):
           for target in self.order[current]:
            if self.Ids[target]==[]:
             print " recursion on "+ str(target) + " from "+str(current)+"  "+str(self.Ids[current])+"  "+""
             self.Ids[target]= self.getIdsQuery(current,target)
             ##########################special case: StraightToMerge
             if self.Ids['repkF']!=[] and not self.checkedStraightToMerge:
                 self.Ids['StraightToMerge']=self.getIdsQuery('repkF','repkMF',option="StraightToMerge")
                 self.checkedStraightToMerge=True
                 if self.Ids['repkMF']!=[]:  self.Ids[target]+=self.Ids['StraightToMerge']
                 self.deleteIds('repkF',self.Ids['StraightToMerge'])
             if target=='repkMF' and self.checkedStraightToMerge:
                 self.Ids[target]+=self.Ids['StraightToMerge']
             # drop repkJ/repkMJ if looking for only  StraightToMerge files
             if self.Ids['StraightToMerge']!=[] and self.Ids['StraightToMerge']==self.Ids['repkMF']:
                  self.Ids['repkF']=[]
                  self.Ids['repkMJ']=[]
             ###################### end special case 
             #print "result on "+target+"  ="+str(self.Ids[target])   +"\n"
             self.generateAllIds(target)
        #else:
          #print "no IDS found :( ... exiting!" 
          
    def buildTables(self):
        """
        This method produce the final results executing the queries: print*.
        This method requires the Ids dictionary to be filled. 
        """
        results=[]
        ## print only correct tables (the one which are in the order dictionary)
        keys=[]
        for key in self.Ids:
            if key in self.order.keys():
                keys.append(key)
                
        #print "KEYS="+ str(keys)
        for t in keys:
            resultList=[]
            origSql=getattr(self, 'print'+t[0].upper()+t[1:])
            n=1+ ( len(self.Ids[t]) // self.MAXDIM) 
            for j in range(1,n+1):
             ids=self.Ids[t][((j-1)*self.MAXDIM):(j*self.MAXDIM)]
             if len(ids)>0: 
               sql=origSql %(','.join(ids))
               partResult = self.dbi.processData(sql, {}, conn = self.conn,
                                      transaction = self.transaction)
               resultList+= self.formatDict(partResult)
            results.append( [{'tablename':self.tableNames[t]}]+ resultList)
        return results
     
     
    # this is the query executor for all the getIds queries!!
    def getIdsQuery(self,current, target,option=""):
      """
      This query produce the ids of the 'target' table starting from the 'current' ids [already done]
      """
      resultIds=[]
      origSql=getattr(self, 'get'+target[0].upper()+target[1:]+'IdsBy'+current[0].upper()+current[1:]+option)
      n=1+ ( len(self.Ids[current]) // self.MAXDIM) 
      #print " n== "+str(n)+ "  len="+str(len(self.Ids[currentIds]))
      for j in range(1,n+1):
          #print 'j=='+str(j)
          ids=self.Ids[current][((j-1)*self.MAXDIM):(j*self.MAXDIM)]
          if len(ids)>0: 
           sql=origSql %(','.join(ids))
           result = self.dbi.processData(sql, {}, conn = self.conn,
                                      transaction = self.transaction)
           resultList= self.format(result)
           resultIds +=  [str(object[0]) for object in resultList]
      return resultIds
                 
        
#################### choice of the order
    def strmExprGreaterThenBulks(self):
      """
      This method hold a special case: when the input is a streamer there are 2 options:
      * simple express-stremer tree
      * complete streamer-repack-reco-alco tree
      The decision is made according with the highest number of streamer_id processed between the ones with style='Bulk' or 'Express' 
      """
      #check express or repack/reco/alca layout!
      if self.vtype=='strm':
       try:
        sql= self.sqlStreamerExpress % self.value
        result = self.dbi.processData(sql, {}, conn = self.conn,
                                      transaction = self.transaction)
        expressDict=self.formatDict(result)[0]
        sql= self.sqlStreamerBulk % self.value
        result = self.dbi.processData(sql, {}, conn = self.conn,
                                      transaction = self.transaction)
        bulkDict=self.formatDict(result)[0]
        bulk=bulkDict['streamer_id']
        express=expressDict['streamer_id']
        #print " values: expr="+str(express)+ "  bulk="+str(bulk)
        if express>0:
          #print str(self.order)
          return True
        if bulk>0:
          return False
       except Exception,e :
         print  e
         pass
       return False

          
################ special methods (used seldomly)
    def deleteIds(self, index, forbidden):
        res=[]
        for i in self.Ids[index]:
            if i not in forbidden:
                res.append(i)
        self.Ids[index]=res      
      
    sqlStreamerExpress=""" SELECT
                count(distinct s.streamer_id) as streamer_id
                from 
                streamer s
                INNER JOIN run_stream_style_assoc rssa ON (rssa.stream_id = s.stream_id) 
                where s.streamer_id = %s AND rssa.style_id = (SELECT id FROM PROCESSING_STYLE WHERE name = 'Express' )"""
    sqlStreamerBulk= """ SELECT
                count(distinct s.streamer_id) as streamer_id
                from 
                streamer s
                INNER JOIN run_stream_style_assoc rssa ON (rssa.stream_id = s.stream_id) 
                where s.streamer_id = %s AND rssa.style_id = (SELECT id FROM PROCESSING_STYLE WHERE name = 'Bulk' )"""
##################################################################### IDS sql EXPR-STRM [8 queries]
    getExprJIdsByStrm="""SELECT  DISTINCT 
  express_job_def.job_id                                
FROM
  express_job_def
WHERE
  express_job_def.job_id IN
  (SELECT DISTINCT
    (express_job_streamer_assoc.job_id) AS distinct_1
  FROM
    express_job_streamer_assoc
  WHERE
    express_job_streamer_assoc.streamer_id IN (%s)
  )
ORDER BY
  express_job_def.job_id DESC"""
  
    getExprFIdsByStrm="""SELECT DISTINCT f.id  
            FROM wmbs_file_details f  
            INNER JOIN repack_streamer_assoc a ON (a.repacked_id = f.id) 
            WHERE  a.streamer_id IN (%s) 
            ORDER BY  1 DESC """
            
            
    getExprMJIdsByExprF="""SELECT j.id
           FROM wmbs_job j 
               INNER JOIN wmbs_job_assoc ja ON (j.id = ja.job) 
               WHERE ja.fileid IN (%s)
               ORDER BY  1 DESC """
               
    getExprMFIdsByExprF="""  SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_fileset_files fsf ON (f.id  = fsf.fileid)
INNER JOIN wmbs_fileset fs        ON (fs.id = fsf.fileset)
INNER JOIN wmbs_file_parent fp    ON (f.id  = fp.child)
WHERE
  fs.name      = 'ExpressDBSUploadable'
AND fp.parent IN (%s)"""
   
    getStrmIdsByExprJ="""
SELECT  DISTINCT 
  streamer.streamer_id 
FROM
  streamer
WHERE
  streamer.streamer_id IN
  (SELECT DISTINCT
    (express_job_streamer_assoc.streamer_id) AS distinct_1
  FROM
    express_job_streamer_assoc
  WHERE
    express_job_streamer_assoc.job_id IN (%s)
  )
ORDER BY
  streamer.streamer_id DESC"""
   
    getStrmIdsByExprF="""SELECT  DISTINCT 
  streamer.streamer_id
FROM
  streamer
WHERE
  streamer.streamer_id IN
  (SELECT DISTINCT
    (repack_streamer_assoc.streamer_id) AS distinct_1
  FROM
    repack_streamer_assoc
  WHERE
    repack_streamer_assoc.repacked_id IN (%s)
  )
ORDER BY
  streamer.streamer_id DESC"""
  
  
    getExprMJIdsByExprMF="""SELECT  DISTINCT 
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja       ON (j.id      = ja.job)
WHERE
  ja.job IN
  (SELECT
    job
  FROM
    wmbs_job_assoc
  INNER JOIN wmbs_file_parent ON (parent = fileid)
  WHERE
    child IN (%s)
  )
ORDER BY
  1 DESC"""
  
    getExprFIdsByExprMJ="""  SELECT DISTINCT f.id  
            FROM wmbs_file_details f  
            INNER JOIN wmbs_job_assoc ja ON (ja.fileid = f.id) WHERE ja.job IN (%s) 
            ORDER BY  1 DESC"""
            
###################################################################### STREAMER
    printStrm="""SELECT  DISTINCT 
  streamer.streamer_id AS streamer_id,
  streamer.lumi_id     AS lumi_id,
  streamer.filesize    AS filesize,
  streamer.events      AS events,
  stream.name          AS stream,
  streamer.lfn         AS lfn
FROM
  streamer
JOIN stream ON streamer.stream_id = stream.id
WHERE
  streamer.streamer_id IN (%s)
ORDER BY
  streamer.streamer_id DESC """
####################################################################### EXPRESS OUTPUT queries

    printExprJ="""SELECT  DISTINCT 
  jd.job_id                                  AS job_id,
  s.run_id                                   AS run_id,
  jd.definition_time                         AS def_time,
  COUNT(DISTINCT(a.streamer_id)) AS num_streamers
FROM
  express_job_def jd
JOIN express_job_streamer_assoc a      ON jd.job_id = a.job_id
JOIN streamer s                   ON s.streamer_id   = a.streamer_id 
WHERE
  jd.job_id IN (%s)
GROUP BY
  jd.job_id,
  jd.definition_time,
  s.run_id
ORDER BY
  jd.job_id DESC """
  
    printExprF="""SELECT DISTINCT f.id, f.filesize, f.events, f.lfn  
            FROM wmbs_file_details f  
            INNER JOIN wmbs_fileset_files fsf ON (f.id = fsf.fileid)
            INNER JOIN wmbs_fileset fs ON (fs.id = fsf.fileset) 
            WHERE  fs.name = 'ExpressMergeable' and f.id IN (%s) 
            ORDER BY  1 DESC """
    
    printExprMJ="""SELECT   DISTINCT j.id as job_id, m.run as run_id, j.state_time as def_time, COUNT(DISTINCT ja.fileid) as inputfiles
           FROM wmbs_job j 
               INNER JOIN wmbs_job_assoc ja ON (j.id = ja.job) 
               INNER JOIN wmbs_file_details f ON (f.id = ja.fileid)
               INNER JOIN wmbs_file_runlumi_map m ON (f.id = m.fileid)
               WHERE j.id IN (%s)
               GROUP BY j.id, m.run,j.state_time    
               ORDER BY  1 DESC"""
    

    printExprMF="""SELECT DISTINCT f.id, f.filesize, f.events, f.lfn  
            FROM wmbs_file_details f  
            INNER JOIN wmbs_file_runlumi_map m ON (f.id = m.fileid)
            INNER JOIN wmbs_fileset_files fsf ON (f.id = fsf.fileid)
            INNER JOIN wmbs_fileset fs ON (fs.id = fsf.fileset) 
            WHERE f.id IN (%s) and fs.name = 'ExpressDBSUploadable' 
            ORDER BY  1 DESC """
#################################################################################################################### REPK-STRM IDS 


    getRepkJIdsByRepkF="""SELECT DISTINCT
    repack_job_streamer_assoc.job_id 
  FROM
    repack_job_streamer_assoc
  WHERE
    repack_job_streamer_assoc.streamer_id IN
    (SELECT
      repack_streamer_assoc.streamer_id
    FROM
      repack_streamer_assoc
    WHERE
      repack_streamer_assoc.repacked_id IN (%s)
    )
 """
    getRepkJIdsByRepkMF=getRepkJIdsByRepkF
    
    getRepkFIdsByStrm="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN repack_streamer_assoc a        ON (a.repacked_id      = f.id)
WHERE
  dt.name          = 'RAW'
AND a.streamer_id IN (%s)
 """
  
  
    getRepkJIdsByStrm="""SELECT  DISTINCT 
  repack_job_def.job_id                      
FROM
  repack_job_def
WHERE
  repack_job_def.job_id IN
  (SELECT DISTINCT
    (repack_job_streamer_assoc.job_id) AS distinct_1
  FROM
    repack_job_streamer_assoc
  WHERE
    repack_job_streamer_assoc.streamer_id IN (%s)
  )
ORDER BY
  repack_job_def.job_id DESC"""
  
  
    getStrmIdsByRepkJ="""SELECT  DISTINCT 
  streamer.streamer_id 
FROM
  streamer
WHERE
  streamer.streamer_id IN
  (SELECT DISTINCT
    (repack_job_streamer_assoc.streamer_id) AS distinct_1
  FROM
    repack_job_streamer_assoc
  WHERE
    repack_job_streamer_assoc.job_id IN (%s)
  )
ORDER BY
  streamer.streamer_id DESC"""
  
    getRepkMJIdsByRepkF="""SELECT DISTINCT
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job)
  WHERE ja.fileid IN (%s)
"""
  #INNER JOIN  wmbs_fileset_files fsf         ON (ja.fileid               = fsf.fileid)
  #INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset AND fs.name = 'Mergeable')
    getRepkMFIdsByRepkF="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.child)
WHERE
  dt.name      = 'RAW'
AND fs.name    = 'DBSUploadable'
AND fp.parent IN (%s)
ORDER BY
  1 DESC"""
    getRepkFIdsByRepkMJ="""SELECT DISTINCT
  f.id
  FROM  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
WHERE
  dt.name   = 'RAW'
AND ja.job IN (%s)
ORDER BY
  1 DESC """
    getRepkMJIdsByRepkMF="""SELECT  DISTINCT 
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job)
WHERE
  ja.job IN
  (SELECT
    job
  FROM
    wmbs_job_assoc
  INNER JOIN wmbs_file_parent ON (parent = fileid)
  WHERE
    child IN (%s)
  )
ORDER BY
  1 DESC """
  
    getRepkMFIdsByRepkFStraightToMerge="""WITH repackTemporaryFileIds AS
(SELECT f.id FROM
 wmbs_file_details f
 WHERE f.id IN (%s))

SELECT  DISTINCT  r.id FROM repackTemporaryFileIds r
MINUS
SELECT DISTINCT
 f.id
FROM
   wmbs_file_details f
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
WHERE
  dt.name   = 'RAW' AND fs.name = 'Mergeable'
AND f.id   IN (SELECT id FROM repackTemporaryFileIds)
ORDER BY
  1 DESC """
  ################################################################################################### REPACK print
  
    printRepkJ="""SELECT  DISTINCT 
       repack_job_def.job_id                                  AS job_id,
       streamer.run_id                                        AS run_id,
       repack_job_def.definition_time                         AS def_time,
      COUNT(DISTINCT( repack_job_streamer_assoc.streamer_id)) AS num_streamers,
       stream.name                                            AS stream,
       job_status.status                                      AS status
    FROM  repack_job_def
    JOIN  job_status                ON  repack_job_def.job_status =  job_status.id
    JOIN  repack_job_streamer_assoc ON  repack_job_def.job_id     =  repack_job_streamer_assoc.job_id
    JOIN  streamer                  ON  streamer.streamer_id      =  repack_job_streamer_assoc.streamer_id
    JOIN  stream                    ON  streamer.stream_id        =  stream.id
    WHERE
        repack_job_def.job_id IN (%s)
    GROUP BY
       repack_job_def.job_id,
       repack_job_def.definition_time,
       streamer.run_id,
       stream.name,
       job_status.status
    ORDER BY
       repack_job_def.job_id DESC """
  
    printRepkF="""SELECT DISTINCT
  f.id, f.filesize,  f.events,  pd.name as dataset,  f.lfn
FROM
   wmbs_file_details f
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN  wmbs_file_runlumi_map m        ON (f.id               = m.fileid)
INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
WHERE
  dt.name   = 'RAW' AND fs.name = 'Mergeable'
AND f.id   IN (%s)
ORDER BY
  1 DESC"""
  
  
    printRepkMF="""SELECT DISTINCT
    f.id,
    f.filesize,
    f.events,
    pd.name as dataset,
    f.lfn
    FROM
    wmbs_file_details f
    INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
    INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
    INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
    INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
    INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
    INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
    WHERE
    dt.name      = 'RAW'
    AND fs.name    = 'DBSUploadable'
    AND f.id IN (%s) 
    ORDER BY
    1 DESC """
  #INNER JOIN  wmbs_file_parent fp            ON (f.id               = fp.child)
  
    printRepkMJ="""SELECT  DISTINCT 
  j.id as job_id,
  m.run as run_id,
  COUNT(DISTINCT ja.fileid) as inputfiles,
  pd.name as dataset
FROM
   wmbs_job j
INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
WHERE
  ja.job IN (%s)
GROUP BY j.id, m.run, pd.name
 """
############################################################################## RECO  IDs

    getRecoJIdsByRepkMF="""SELECT  DISTINCT j.id
FROM  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job)
WHERE  ja.job IN
  (SELECT     job
  FROM    wmbs_job_assoc
  WHERE fileid IN (%s))
ORDER BY 1 DESC """

    getRecoFIdsByRepkMF="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.child)
WHERE ((dt.name                                                              ='RECO'
AND fs.name                                                                  = 'Mergeable')
OR (dt.name                                                                  ='ALCARECO'
AND fs.name                                                                  = 'AlcaSkimmable')
OR (dt.name                                                                  ='AOD')
OR (dt.name                                                                  ='DQM'))
AND fp.parent                                                               IN (%s)
ORDER BY
  1 DESC """

    getRecoMJIdsByRecoF="""SELECT DISTINCT  j.id
FROM  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job)
INNER JOIN wmbs_file_details f            ON (ja.fileid = f.id)
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
INNER JOIN dataset_path dp                ON (dp.id     = a.dataset_path_id)
INNER JOIN primary_dataset pd             ON (pd.id     = dp.primary_dataset)
INNER JOIN data_tier dt                   ON (dt.id = dp.data_tier)
WHERE  
(dt.name      = 'RECO' OR dt.name      = 'AOD' OR dt.name      = 'DQM') 
AND 
ja.job IN
  (SELECT    job 
  FROM    wmbs_job_assoc
  WHERE   fileid IN (%s))
ORDER BY
  1 DESC """ 

    getRecoMFIdsByRecoF="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.child)
WHERE
  ((dt.name      = 'RECO'
AND fs.name    = 'DBSUploadable')
OR (dt.name                                                                  ='AOD')
OR (dt.name                                                                  ='DQM'))
AND fp.parent IN (%s)
ORDER BY
  1 DESC """
  
    getRepkMFIdsByRecoJ="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
WHERE
  dt.name   = 'RAW'
AND fs.name = 'DBSUploadable'
AND ja.job IN (%s)
ORDER BY
  1 DESC """
  
    getRecoJIdsByRecoF="""SELECT   DISTINCT  j.id
FROM  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job)
WHERE
  ja.job IN
  (SELECT     job
  FROM    wmbs_job_assoc
   INNER JOIN wmbs_file_parent fp ON (fp.parent = fileid)
   INNER JOIN wmbs_job_assoc ja ON (ja.fileid = fp.parent) 
   INNER JOIN wmbs_job_mask jm ON ( jm.job = ja.job ) 
   INNER JOIN wmbs_file_details fd ON ( fd.id = fp.child AND jm.firstevent = fd.first_event ) 
  WHERE
    child IN (%s)
  )
ORDER BY
  1 DESC
    """
    getRecoFIdsByRecoMJ="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
WHERE ((dt.name                             ='RECO'
AND fs.name                                = 'Mergeable')
OR (dt.name                                ='ALCARECO'
AND fs.name                                = 'AlcaSkimmable')
OR (dt.name                                                                  ='AOD')
OR (dt.name                                                                  ='DQM'))
AND ja.job                                 IN (%s)
ORDER BY
  1 DESC """
  
    getRecoFIdsByRecoMF="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.parent)
WHERE ((dt.name                                                              ='RECO'
AND fs.name                                                                  = 'Mergeable')
OR (dt.name                                                                  ='ALCARECO'
AND fs.name                                                                  = 'AlcaSkimmable')
OR (dt.name                                                                  ='AOD')
OR (dt.name                                                                  ='DQM'))
AND fp.child                                                                IN (%s)
ORDER BY
  1 DESC """
    getRecoFIdsByRecoJ="""
SELECT DISTINCT fp.child
FROM wmbs_file_parent fp 
INNER JOIN wmbs_job_assoc ja ON (ja.fileid = fp.parent) 
INNER JOIN wmbs_job_mask jm ON ( jm.job = ja.job ) 
INNER JOIN wmbs_file_details fd ON ( fd.id = fp.child AND jm.firstevent = fd.first_event ) 
WHERE ja.job IN (%s)
 """
    getRecoFIdsByAlcaJ=""" SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
WHERE (dt.name                                                               ='ALCARECO'
AND fs.name                                                                  = 'AlcaSkimmable')
AND ja.job                                                                  IN (%s)
ORDER BY
  1 DESC"""
#########################################################################################################3 RECO print


    printRecoF="""SELECT DISTINCT
  f.id,
  f.filesize,
  f.events,
  dt.name AS tier,
  pd.name AS dataset,
  f.lfn
FROM
   wmbs_file_details f
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
WHERE ((dt.name                                                              ='RECO'
AND fs.name                                                                  = 'Mergeable')
OR (dt.name                                                                  ='ALCARECO'
AND fs.name                                                                  = 'AlcaSkimmable')
OR (dt.name                                                                  ='AOD')
OR (dt.name                                                                  ='DQM'))
AND f.id                                                                    IN (%s)
ORDER BY
  1 DESC """
  
    printRecoJ="""SELECT
  j.id as job_id,
  m.run as run_id,
  j.state_time as def_time,
  COUNT(DISTINCT ja.fileid) as inputfiles,
  pd.name as dataset
FROM
   wmbs_job j
INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
WHERE
  ja.job IN (%s)
GROUP BY
  j.id,
  m.run,
  j.state_time,
  pd.name
ORDER BY
  1 DESC """
    printRecoMJ=""" SELECT
  j.id as job_id,
  m.run as run_id,
  j.state_time as def_time,
  dt.name AS tier,
  COUNT(DISTINCT ja.fileid) as inputfiles,
  pd.name as dataset
FROM
   wmbs_job j
INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
WHERE
  ja.job IN (%s)
GROUP BY
  j.id,
  m.run,
  state_time,
  dt.name,
  pd.name
ORDER BY
  1 DESC """
    printRecoMF=""" SELECT DISTINCT
  f.id,
  f.filesize,
  f.events,
  dt.name AS tier,
  pd.name as dataset,
  f.lfn
FROM
   wmbs_file_details f
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN  wmbs_file_parent fp            ON (f.id               = fp.child)
WHERE
  ((dt.name      = 'RECO'
AND fs.name    = 'DBSUploadable') 
OR (dt.name      ='DQM')
OR (dt.name      ='AOD'))
AND f.id IN (%s)
ORDER BY
  1 DESC"""
  
############################################################################################### ALCA IDS
    getAlcaJIdsByRecoF="""SELECT  DISTINCT 
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
WHERE
  ja.job IN
  ( SELECT job FROM wmbs_job_assoc WHERE fileid IN (%s)
  )
AND dt.name ='ALCARECO'
AND fs.name = 'AlcaSkimmable'
ORDER BY
  1 DESC """
    getAlcaJIdsByAlcaF="""SELECT  DISTINCT 
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
WHERE
  ja.job IN
  (SELECT
    job
  FROM
    wmbs_job_assoc
  INNER JOIN wmbs_file_parent ON (parent = fileid)
  WHERE
    child IN (%s)
  )
ORDER BY
  1 DESC """
    getAlcaFIdsByRecoF="""SELECT DISTINCT
  f.id
FROM
 wmbs_file_details f
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN  dataset_path dp                ON (a.dataset_path_id    = dp.id)
INNER JOIN  primary_dataset pd             ON (dp.primary_dataset   = pd.id)
INNER JOIN  data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN  wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN  wmbs_fileset fs                ON (fs.id                = fsf.fileset)
INNER JOIN  processed_dataset pds          ON (dp.processed_dataset = pds.id)
INNER JOIN  wmbs_file_parent fp            ON (f.id                 = fp.child)
WHERE fp.parent IN (%s) AND
 dt.name   = 'ALCARECO'
AND fs.name = 'Mergeable'
ORDER BY
  1 DESC"""

    getAlcaMJIdsByAlcaF="""SELECT  DISTINCT 
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
WHERE
  ja.job IN
  (SELECT
    job
  FROM
    wmbs_job_assoc
  WHERE
    fileid IN (%s)
  )
ORDER BY
  1 DESC """
    getAlcaMFIdsByAlcaF="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id    = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset   = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id                 = fp.child)
WHERE
  dt.name      = 'ALCARECO'
AND fs.name    = 'DBSUploadable'
AND fp.parent IN (%s)
ORDER BY
  1 DESC """
    getAlcaFIdsByAlcaJ="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
WHERE (dt.name                                                               ='ALCARECO'
AND fs.name                                                                  = 'AlcaSkimmable')
AND ja.job                                                                  IN (%s)
ORDER BY
  1 DESC """
    getAlcaFIdsByAlcaMJ="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id    = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset   = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN wmbs_job_assoc ja              ON (ja.fileid            = f.id)
WHERE
  dt.name   = 'ALCARECO'
AND ja.job IN (%s)
ORDER BY
  1 DESC """
    getAlcaFIdsByAlcaMF="""SELECT DISTINCT
  f.id
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id    = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset   = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id                 = fp.parent)
WHERE
  dt.name     = 'ALCARECO'
AND fs.name   = 'Mergeable'
AND fp.child IN (%s)
ORDER BY
  1 DESC """
  
    getAlcaMJIdsByAlcaMF="""SELECT distinct
  j.id
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
INNER JOIN processed_dataset pds          ON (dp.processed_dataset = pds.id)
WHERE
  ja.job IN
  (SELECT
    job
  FROM
    wmbs_job_assoc
  INNER JOIN wmbs_file_parent ON (parent = fileid)
  WHERE
    child IN (%s)
  )
ORDER BY
  1 DESC"""
  
  ######################################################################################################## ALCA print
  
    printAlcaMF=""" SELECT DISTINCT
  f.id,
  f.filesize,
  f.events,
  pd.name as dataset,
  f.lfn,
  pds.name AS skim
FROM
  wmbs_file_details f
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (a.dataset_path_id    = dp.id)
INNER JOIN primary_dataset pd             ON (dp.primary_dataset   = pd.id)
INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
INNER JOIN wmbs_file_parent fp            ON (f.id                 = fp.child)
INNER JOIN processed_dataset pds          ON (dp.processed_dataset = pds.id)
WHERE
  dt.name      = 'ALCARECO'
AND fs.name    = 'DBSUploadable'
AND f.id IN (%s)
ORDER BY
  1 DESC"""
    printAlcaF=""" SELECT DISTINCT
  f.id,
  f.filesize,
  f.events,
  pd.name as dataset,
  f.lfn,
  pds.name AS skim
FROM
   wmbs_file_details f
INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN  dataset_path dp                ON (a.dataset_path_id    = dp.id)
INNER JOIN  primary_dataset pd             ON (dp.primary_dataset   = pd.id)
INNER JOIN  data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN  wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN  wmbs_fileset fs                ON (fs.id                = fsf.fileset)
INNER JOIN  processed_dataset pds          ON (dp.processed_dataset = pds.id)
WHERE
  dt.name   = 'ALCARECO'
AND fs.name = 'Mergeable'
AND f.id   IN (%s)
ORDER BY
  1 DESC"""
  
  
  
    printAlcaJ=""" SELECT  DISTINCT 
  j.id as job_id,
  m.run as run_id,
  j.state_time as def_time,
  COUNT(DISTINCT ja.fileid) as inputfiles,
  pd.name as dataset
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
INNER JOIN processed_dataset PROC         ON (dp.processed_dataset = proc.id)
WHERE
  ja.job IN (%s)
AND dt.name ='ALCARECO'
AND fs.name = 'AlcaSkimmable'
GROUP BY
  j.id,
  m.run,
  j.state_time,
  pd.name,
  proc.name
ORDER BY
  1 DESC"""
    
    printAlcaMJ=""" SELECT  DISTINCT 
  j.id as job_id,
  m.run as run_id,
  j.state_time as def_time,
  COUNT(DISTINCT ja.fileid) as inputfiles,
  pd.name as dataset,
  pds.name AS skim
FROM
  wmbs_job j
INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
INNER JOIN processed_dataset pds          ON (dp.processed_dataset = pds.id)
WHERE
  ja.job IN (%s)
GROUP BY
  j.id,
  m.run,
  j.state_time,
  pd.name,
  pds.name
ORDER BY
  1 DESC"""
    ## query executor for the tables generation!!
    #def execTablesQueries(self,current,tablenames,sqlvarnames):
      #bindVars={}
      #results=[]
      #currentIds=current+"Ids"
      #if len(self.Ids[currentIds])==0:
        ##print "No base ID "
        #return [{'tablename':tablenames[i]} for i in range(len(tablenames))]
      
      ##find the query witch produce the table of the input value ---> must be bounded
      ##inputTableSql="sql"+self.vtype[0].upper()+self.vtype[1:4]
      ##if self.vtype[len(self.vtype)-2]=='m': 
          ##inputTableSql+='Merge'
      ##if self.vtype[len(self.vtype)-1]=='j':
          ##inputTableSql+='Jobs'
      ##elif self.vtype[len(self.vtype)-1]=='f':
          ##inputTableSql+='Files'
      ##inputTableSql+='by'+self.vtype[0].upper()+self.vtype[1:4]+"Ids"
      ##print "inputTableSql=="+inputTableSql
      
      #for i in range(len(tablenames)):
        #bindVars={}
        #origSql=getattr(self,sqlvarnames[i])    
        ##print tablenames[i] +"   " +sql
        ##if tablenames[i] == self.limitTable:               
           ##sql= self.sqlPrefix + sql + self.sqlSuffix
           ##bindVars={'uplimit':self.uplimit,'downlimit':self.downlimit}
        #result=[]
        #n=1+ ( len(self.Ids[currentIds]) // self.MAXDIM) 
        ##print " n== "+str(n)+ "  len="+str(len(self.Ids[currentIds]))
        #for j in range(1,n+1):
          ##print 'j=='+str(j)
          #ids=self.Ids[currentIds][((j-1)*self.MAXDIM):(j*self.MAXDIM)]
          #if len(ids)>0:
           ##if origSql==inputTableSql:  use f.id or j.id as clausue 
           #sql=origSql %(','.join(ids))
           ##else: sql=origSql %(','.join(ids),"")
           ##print sql
           #partRes = self.dbi.processData(sql, bindVars, conn = self.conn,
                                      #transaction = self.transaction)
           #result+=self.formatDict(partRes)
          ##print str(result) +"\n\n"
        #results.append( [{'tablename':tablenames[i]}]+ result)
      #return results
      
######################################################################################

    #def getJobConnections(self):
           #if self.order== self.orderRepk and self.Ids['RepkIds']==[] and self.vtype=='repkj':
               #target='Strm'
               #tableNamePrefix="Repack"
               #IdsSql= self.sqlStrmIdsByRepkJob%(self.value)
               #sql= self.sqlRepkJobsByItsIds%(self.value)
           #if self.order== self.orderExpr and self.Ids['ExprIds']==[] and  self.vtype=='exprj':
               #target='Strm'
               #tableNamePrefix="Express"
               #IdsSql= self.sqlStrmIdsByExpressJob%(self.value)
               #sql= self.sqlExprJobsByItsIds%(self.value)
           #if self.order== self.orderRepk and self.Ids['RecoIds']==[] and self.vtype=='recoj' :
               #target='Repk'
               #tableNamePrefix="Reco"
               #IdsSql= self.sqlRepkIdsbyRecoJob%(self.value)
               #sql= self.sqlRecoJobsByItsIds%(self.value)
           #if  self.order== self.orderRepk and self.Ids['AlcaIds']==[] and self.vtype=='alcaj' :
               #target='Reco'
               #tableNamePrefix="Alca"
               #IdsSql= self.sqlRecoIdsByAlcaJob%(self.value)
               #sql= self.sqlAlcaJobsByItsIds%(self.value)
           #try:
             #return (target,tableNamePrefix,IdsSql,sql)
           #except:
             #return None

    #def singleJobTable(self,tableNamePrefix, sql):
        #res=[]
        #partRes = self.dbi.processData(sql, {}, conn = self.conn, transaction = self.transaction)
        #res.append( [{'tablename': tableNamePrefix+'Jobs'}]+ self.formatDict(partRes))
        #res.append( [{'tablename':tableNamePrefix+'MergeJobs'}])
        #res.append( [{'tablename':tableNamePrefix+'Files'}])
        #res.append( [{'tablename':tableNamePrefix+'MergeFiles'}])
        #return res
    #def addNextJobs(self,current):
        #res=[]
        #if self.order== self.orderExpr and current=="Expr" and self.Ids['StrmIds']!=[] :
           #tableNamePrefix="Express"
           #sql=self.sqlExprJobByStrmIds%(','.join(self.Ids['StrmIds']))
        #if self.order== self.orderRepk and current=="Repk" and self.Ids['StrmIds']!=[] :
           #tableNamePrefix="Repack"
           #sql=self.sqlRepkJobByStrmIds%(','.join(self.Ids['StrmIds']))
        #if self.order== self.orderRepk and current=="Repk" and self.Ids['RepkIds']!=[] :
           #tableNamePrefix="Reco"
           #sql=self.sqlRecoJobByRepkIds%(','.join(self.Ids['RepkIds']))  
        #if self.order== self.orderRepk and current=="Reco" and self.Ids['RecoIds']!=[] :
           #tableNamePrefix="Alca"
           #sql=self.sqlAlcaJobByRecoIds%(','.join(self.Ids['RecoIds']))
        #try:
          #partRes = self.dbi.processData(sql, {}, conn = self.conn, transaction = self.transaction)  
          #res.append( [{'tablename': tableNamePrefix+'Jobs'}]+ self.formatDict(partRes))
          #res.append( [{'tablename':tableNamePrefix+'MergeJobs'}])
          #res.append( [{'tablename':tableNamePrefix+'Files'}])
          #res.append( [{'tablename':tableNamePrefix+'MergeFiles'}])
          #return res
        #except Exception, e:
          #print e
          #return res  
######################################################333
# some request are not correctly managed by the system.
# this is the place for such "exceptions"
# this code is OUT OF MAIN STRUCTURE!!!
# FIX ME: find a proper place and write a better code for this part
    #def exceptionManager(self,target):
       #print "TARGET=="+target
       #res=[]
       #print "handling Excepion!!! " + str(self.vtype)+ "  "+ str(self.value)
       
       ##jobs without files-IDs 
       #if self.vtype!="strm" and self.vtype[4]=='j':
          
           #try:
            #print "TARGET="+target
            #self.Ids[target]=self.execGetIdsQuery(IdsSql)
            #print target+"==="+str(self.Ids[target])
            #if self.Ids[target]==[]: return res
            
           #except Exception, e:
                #print e
                #return res
       #else:
           ####not others???
         
       
       ##BUGFIX: Reco Jobs may exists even if Reco Files doesn't (1)
       ##if self.vtype=="recoj" and self.Ids['RepkIds']==[] and  self.Ids['RecoIds']==[]:
         #### build single recoj table
         ##sql= self.sqlRecoJobsByItsIds%(self.value)
         ##partRes = self.dbi.processData(sql, {}, conn = self.conn, transaction = self.transaction)
         ##res.append( [{'tablename':'RecoJobs'}]+ self.formatDict(partRes))
         ##res.append( [{'tablename':'RecoMergeJobs'}])
         ##res.append( [{'tablename':'RecoFiles'}])
         ##res.append( [{'tablename':'RecoMergeFiles'}])
         ##### calcolate repakMergeFiles Ids
         ##sql= self.sqlRepkIdsByRecoJobs%(self.value)
         ##self.Ids['repkIds']=self.execGetIdsQuery(sql)
         ##if len(self.Ids['repkIds'])>0:
         ###print "REPK IDS==="+str(self.Ids['repkIds'])
         ###self.order={'Strm': ['Repk'], 'Repk':['Strm']}
         ###res+= self.generateResults('Repk')
          ##res+=self.buildRepkResults('repk')
          ##res+=self.buildStrmResults('repk')
         ##else: 
          ##res.append( [{'tablename':'RepackJobs'}])
          ##res.append( [{'tablename':'RepackMergeJobs'}])
          ##res.append( [{'tablename':'RepackFiles'}])
          ##res.append( [{'tablename':'RepackMergeFiles'}])  
       ###BUGFIX: Reco Jobs may exists even if Reco Files doesn't (2)
       ##elif self.Ids['RepkIds']!=[] and self.Ids['RecoIds']==[]:
         ##sql= self.sqlRecoJobsByRepakIds%( ','.join(self.Ids['RepkIds']))
         ##partRes = self.dbi.processData(sql, {}, conn = self.conn, transaction = self.transaction)
         ##res.append( [{'tablename':'RecoJobs'}]+ self.formatDict(partRes))
         ##res.append( [{'tablename':'RecoMergeJobs'}])
         ##res.append( [{'tablename':'RecoFiles'}])
         ##res.append( [{'tablename':'RecoMergeFiles'}])
       ###BUGFIX: if ONLY streamers table is present I must add some empty one
       ##elif self.Ids['RepkIds']==[] and self.Ids['StrmIds']!=[] and self.Ids['ExprIds']==[] and self.vtype!="repkj" :
         ###res.append( [{'tablename':'ExpressJobs'}])
         ###res.append( [{'tablename':'ExpressMergeJobs'}])
         ###res.append( [{'tablename':'ExpressFiles'}])
         ###res.append( [{'tablename':'ExpressMergeFiles'}])
       ###elif self.vtype=="repkj" and self.Ids['RepkIds']==[]:
           ####++++ DO THE OPPOSITE!!
         ###sql= self.sqlStrmByRepakJob%(self.value)
         ###self.Ids['StrmIds']=self.execGetIdsQuery(sql)
         ####print "STRM="+str(self.Ids['StrmIds'])
         ###self.order={'Strm': []}
         ###res= self.generateResults('Strm')
         ###sql= self.sqlRepkJobByRepakJob%(self.value)
         ###partRes = self.dbi.processData(sql, {}, conn = self.conn, transaction = self.transaction)
         ####print "RES="+ str(res)
         ###res.append( [{'tablename':'RepackJobs'}]+ self.formatDict(partRes))
         ###res.append( [{'tablename':'RepackMergeJobs'}])
         ###res.append( [{'tablename':'RepackFiles'}])
         ###res.append( [{'tablename':'RepackMergeFiles'}])       
       ###elif self.vtype=="exprj" and self.Ids['ExprIds']==[]:
           ####++++ DO THE OPPOSITE!!
         ###sql= self.sqlStrmByExprJob%(self.value)
         ###self.Ids['StrmIds']=self.execGetIdsQuery(sql)
         ####print "STRM="+str(self.Ids['StrmIds'])
         ###res= self.generateResults('Strm')
         ###sql= self.sqlExprJobByExprJob%(self.value)
         ###partRes = self.dbi.processData(sql, {}, conn = self.conn, transaction = self.transaction)
                ####print "RES="+ str(res)
         ###res.append( [{'tablename':'ExpressJobs'}]+ self.formatDict(partRes))
         ###res.append( [{'tablename':'ExpressMergeJobs'}])
         ###res.append( [{'tablename':'ExpressFiles'}])
         ###res.append( [{'tablename':'ExpressMergeFiles'}])
         ###add exception  for  express jobs when they have not yet a file associated yet!
       ##return res
################################################################################################## get*Ids 
    #def getStrmIds(self):
      #bindVars={}
      #if self.Ids['StrmIds']!=[]:
        #return self.Ids['StrmIds'] 
      #elif self.Ids['RepkIds']!=[]:
         #sql=self.sqlStrmIdsByRepk
         #sql=sql %(','.join(self.Ids['RepkIds']))
         ##bindVars={'maxIdNum': str(self.maxIdNum)}
      #elif self.Ids['ExprIds']!=[]:
         #sql=self.sqlStrmIdsByExpr
         #sql=sql %(','.join(self.Ids['ExprIds']))
      #try: 
        #sql+= " "
      #except:
        #raise Exception('impossible to get Stream Ids')
      #return self.execGetIdsQuery(sql,bindVars)

    #def getRepkIds(self):
      #if self.Ids['StrmIds']!=[]:
         #sql=self.sqlRepkIdsByStrm
         #sql=sql %(','.join(self.Ids['StrmIds']))
      #elif self.Ids['RecoIds']!=[]:
         #sql=self.sqlRepkIdsByReco
         #sql=sql %(','.join(self.Ids['RecoIds']))
      #elif self.vtype=='repkf':
        #return [self.value,]
      #elif self.vtype=='repkj':
         #sql=self.sqlRepkIdsByRepkJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='repkmj':
         #sql=self.sqlRepkIdsByRepkMergeJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='repkmf':
         #sql=self.sqlRepkIdsByRepkMergeFile
         #sql=sql %(str(self.value))
      #try: 
        #sql+= " "
      #except:
        #raise Exception('impossible to get Repack Ids')
      #return self.execGetIdsQuery(sql)
      
      
    #def getRecoIds(self):
      #if self.Ids['RepkIds']!=[]:
         #sql=self.sqlRecoIdsByRepk
         #sql=sql %(','.join(self.Ids['RepkIds']))
      #elif self.Ids['AlcaIds']!=[]:
         #sql=self.sqlRecoIdsByAlca
         #sql=sql %(','.join(self.Ids['AlcaIds']))
      #elif self.vtype=='recof':
        #return [self.value,]
      #elif self.vtype=='recoj':
         #sql=self.sqlRecoIdsByRecoJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='recomj':
         #sql=self.sqlRecoIdsByRecoMergeJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='recomf':
         #sql=self.sqlRecoIdsByRecoMergeFile
         #sql=sql %(str(self.value))
      #try: 
        #sql+= " "
      #except:
        #raise Exception('impossible to get Reco Ids')
      #return self.execGetIdsQuery(sql)
      
      
    #def getAlcaIds(self):
      #if self.Ids['RecoIds']!=[]:
         #sql=self.sqlAlcaIdsByReco
         #sql=sql %(','.join(self.Ids['RecoIds']))
      #elif self.vtype=='alcaf':
        #return [self.value,]
      #elif self.vtype=='alcaj':
         #sql=self.sqlAlcaIdsByAlcaJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='alcamj':
         #sql=self.sqlAlcaIdsByAlcaMergeJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='alcamf':
         #sql=self.sqlAlcaIdsByAlcaMergeFile
         #sql=sql %(str(self.value))
      #try: 
        #sql+= " "
      #except:
        #raise Exception('impossible to get Alca Ids')
      #return self.execGetIdsQuery(sql)

            
    #def getExprIds(self):
      #self.order=self.orderExpr
      #if self.Ids['StrmIds']!=[]:
         #sql=self.sqlExprIdsByStrm
         #sql=sql %(','.join(self.Ids['StrmIds']))
      #elif self.vtype=='exprf':
        #return [self.value,]
      #elif self.vtype=='exprj':
         #sql=self.sqlExprIdsByExprJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='exprmj':
         #sql=self.sqlExprIdsByExprMergeJob
         #sql=sql %(str(self.value))
      #elif self.vtype=='exprmf':
         #sql=self.sqlExprIdsByExprMergeFile
         #sql=sql %(str(self.value))
      #try: 
        #sql+= " "
      #except:
        #raise Exception('impossible to get Express Ids')
      #return self.execGetIdsQuery(sql)

         
############################################################################################################################ build*results

    #def buildAlcaResults(self,current):
      #tablenames=self.tableNames[9:13]
      #sqlvarnames=['sqlAlcaFilesByAlcaIds','sqlAlcaJobsByAlcaIds','sqlAlcaMergeJobsByAlcaIds','sqlAlcaMergeFilesByAlcaIds']
      #return self.execTablesQueries(current,tablenames,sqlvarnames)

    #def buildRecoResults(self,current):
      #tablenames=self.tableNames[5:9]
      #sqlvarnames=['sqlRecoFilesByRecoIds','sqlRecoJobsByRecoIds','sqlRecoMergeJobsByRecoIds','sqlRecoMergeFilesByRecoIds']
      #return self.execTablesQueries(current,tablenames,sqlvarnames)

    #def buildRepkResults(self,current):
      #tablenames=self.tableNames[1:5]
      #sqlvarnames=['sqlRepkFilesByRepkIds','sqlRepkJobsByRepkIds','sqlRepkMergeJobsByRepkIds','sqlRepkMergeFilesByRepkIds']
      #return self.execTablesQueries(current,tablenames,sqlvarnames)
    
    #def buildExprResults(self,current):
      #tablenames=self.tableNames[13:17]
      #self.order=self.orderExpr
      #sqlvarnames=['sqlExprFilesByExprIds','sqlExprJobsByExprIds','sqlExprMergeJobsByExprIds','sqlExprMergeFilesByExprIds']
      #return self.execTablesQueries(current,tablenames,sqlvarnames)
    
    #def buildStrmResults(self,current):
      ##check express or repack/reco/alca layout!
      #if self.vtype=='strm':
       #try:
        #sql= self.sqlStreamerExpress % self.value
        #result = self.dbi.processData(sql, {}, conn = self.conn,
                                      #transaction = self.transaction)
        #expressDict=self.formatDict(result)[0]
        #sql= self.sqlStreamerBulk % self.value
        #result = self.dbi.processData(sql, {}, conn = self.conn,
                                      #transaction = self.transaction)
        #bulkDict=self.formatDict(result)[0]
        #bulk=bulkDict['streamer_id']
        #express=expressDict['streamer_id']
        ##print " values: expr="+str(express)+ "  bulk="+str(bulk)
        #if express>0:
          ##print str(self.order)
          #self.order=self.orderExpr
       #except Exception,e :
         #print  e
         #pass
      #tablenames=self.tableNames[0:1]
      #sqlvarnames=['sqlStrmByStrmIds',]
      #return self.execTablesQueries(current,tablenames,sqlvarnames)
      
     

#################################################################### SQL
##you can cut and paste  the queries on SQLDeveloeper :)

####################### Services queries
    #sqlPrefix=""" SELECT * FROM    \
  #( SELECT       \
       #a.*,      \
      #ROWNUM AS ora_rn     \
    #FROM        \
      #("""
  
    #sqlSuffix="""  ) a    \
    #WHERE                \
      #ROWNUM <= :uplimit    \
  #) WHERE              \
  #ora_rn > :downlimit """
  
  
  
####################  queries BySameIds
    #sqlStrmByStrmIds="""SELECT
    #streamer.streamer_id AS streamer_id,
    #streamer.lumi_id     AS lumi_id,
    #streamer.filesize    AS filesize,
    #streamer.events      AS events,
    #stream.name          AS stream,
    #streamer.lfn         AS lfn
  #FROM
    #streamer
  #JOIN stream ON streamer.stream_id = stream.id
  #WHERE
    #streamer.streamer_id IN (%s) 
  #ORDER BY
    #streamer.streamer_id DESC """
    ##BUG FIX
##streamer.exportable  AS streamer_exp,    
    #sqlRepkMergeFilesByRepkIds="""SELECT DISTINCT
    #f.id,
    #f.filesize,
    #f.events,
    #pd.name as dataset,
    #f.lfn
    #FROM
    #wmbs_file_details f
    #INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
    #INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
    #INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
    #INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
    #INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
    #INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
    #INNER JOIN  wmbs_file_parent fp            ON (f.id               = fp.child)
    #WHERE
    #dt.name      = 'RAW'
    #AND fs.name    = 'DBSUploadable'
    #AND fp.parent IN (%s) 
    #ORDER BY
    #1 DESC """
  
  
    #sqlRepkMergeJobsByRepkIds="""SELECT
  #j.id as job_id,
  #m.run as run_id,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset
#FROM
   #wmbs_job j
#INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
#INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
#INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
#INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
#INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
#WHERE
  #ja.job IN
  #(SELECT job FROM  wmbs_job_assoc WHERE fileid IN (%s))
#GROUP BY j.id, m.run, pd.name
 #"""
  ##AND ja.fileid IN (%s) )
 
    #sqlRepkFilesByRepkIds="""SELECT DISTINCT
  #f.id, f.filesize,  f.events,  pd.name as dataset,  f.lfn
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
#INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
#INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
#INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
#INNER JOIN  wmbs_file_runlumi_map m        ON (f.id               = m.fileid)
#INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
#INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
#WHERE
  #dt.name   = 'RAW' AND fs.name = 'Mergeable'
#AND f.id   IN (%s)
#ORDER BY
  #1 DESC"""
  ##
  
    #sqlRepkJobsByRepkIds="""SELECT
       #repack_job_def.job_id                                  AS job_id,
       #streamer.run_id                                        AS run_id,
       #repack_job_def.definition_time                         AS def_time,
      #COUNT(DISTINCT( repack_job_streamer_assoc.streamer_id)) AS num_streamers,
       #stream.name                                            AS stream,
       #job_status.status                                      AS status
    #FROM  repack_job_def
    #JOIN  job_status                ON  repack_job_def.job_status =  job_status.id
    #JOIN  repack_job_streamer_assoc ON  repack_job_def.job_id     =  repack_job_streamer_assoc.job_id
    #JOIN  streamer                  ON  streamer.streamer_id      =  repack_job_streamer_assoc.streamer_id
    #JOIN  stream                    ON  streamer.stream_id        =  stream.id
    #WHERE
         #repack_job_streamer_assoc.streamer_id IN
        #(SELECT    repack_streamer_assoc.streamer_id  FROM  repack_streamer_assoc
        #WHERE  repack_streamer_assoc.repacked_id IN (%s))
    #GROUP BY
       #repack_job_def.job_id,
       #repack_job_def.definition_time,
       #streamer.run_id,
       #stream.name,
       #job_status.status
    #ORDER BY
       #repack_job_def.job_id DESC """
       
    #sqlExprJobsByExprIds="""SELECT
  #express_job_def.job_id                                  AS job_id,
  #streamer.run_id                                         AS run_id,
  #express_job_def.definition_time                         AS def_time,
  #COUNT(DISTINCT(express_job_streamer_assoc.streamer_id)) AS num_streamers
#FROM
  #express_job_def
#JOIN express_job_streamer_assoc ON express_job_def.job_id = express_job_streamer_assoc.job_id
#JOIN streamer                   ON streamer.streamer_id   = express_job_streamer_assoc.streamer_id
#WHERE
  #express_job_def.job_id IN
  #( SELECT DISTINCT
    #(express_job_streamer_assoc.job_id) AS distinct_1
  #FROM
    #express_job_streamer_assoc
  #WHERE
    #express_job_streamer_assoc.streamer_id IN
    #(SELECT
      #streamer.streamer_id AS CMS_T0AST_1_streamer_str_1
    #FROM
      #streamer
    #JOIN stream ON streamer.stream_id = stream.id
    #WHERE
      #streamer.streamer_id IN
      #(SELECT DISTINCT
        #(repack_streamer_assoc.streamer_id) AS distinct_1
      #FROM
        #repack_streamer_assoc
      #WHERE
        #repack_streamer_assoc.repacked_id IN (%s)
      #)
    #)
  #)
#GROUP BY
  #express_job_def.job_id,
  #express_job_def.definition_time,
  #streamer.run_id
#ORDER BY
  #express_job_def.job_id DESC"""
  
    #sqlRecoFilesByRecoIds="""SELECT DISTINCT
  #f.id,
  #f.filesize,
  #f.events,
  #dt.name AS tier,
  #pd.name AS dataset,
  #f.lfn
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
#INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
#INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
#INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
#INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
#INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
#WHERE ((dt.name                                                              ='RECO'
#AND fs.name                                                                  = 'Mergeable')
#OR (dt.name                                                                  ='ALCARECO'
#AND fs.name                                                                  = 'AlcaSkimmable'))
#AND f.id                                                                    IN (%s)
#ORDER BY
  #1 DESC """
  
    #sqlRecoJobsByRecoIds="""SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset
#FROM
   #wmbs_job j
#INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
#INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
#INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
#INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
#INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
#WHERE
  #ja.job IN
  #(SELECT
    #job
  #FROM
     #wmbs_job_assoc
  #INNER JOIN  wmbs_file_parent ON (parent = fileid)
  #WHERE
    #child IN (%s)
  #)
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name
#ORDER BY
  #1 DESC """
    #sqlRecoMergeJobsByRecoIds=""" SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset
#FROM
   #wmbs_job j
#INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
#INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
#INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
#INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
#INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
#WHERE
  #ja.job IN
  #(SELECT ja.job FROM  wmbs_job j 
          #INNER JOIN wmbs_job_assoc ja  ON (j.id      = ja.job)
          #INNER JOIN  wmbs_jobgroup jg ON (jg.id = j.jobgroup )
        #WHERE ja.fileid IN (%s)
          #AND jg.subscription = 
 #( SELECT id FROM wmbs_subscription WHERE fileset=( SELECT id FROM wmbs_fileset WHERE name='Mergeable' ) ))
#GROUP BY
  #j.id,
  #m.run,
  #state_time,
  #pd.name
#ORDER BY
  #1 DESC """
    #sqlRecoMergeFilesByRecoIds=""" SELECT DISTINCT
  #f.id,
  #f.filesize,
  #f.events,
  #pd.name as dataset,
  #f.lfn
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
#INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
#INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
#INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
#INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
#INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
#INNER JOIN  wmbs_file_parent fp            ON (f.id               = fp.child)
#WHERE
  #dt.name      = 'RECO'
#AND fs.name    = 'DBSUploadable'
#AND fp.parent IN (%s)
#ORDER BY
  #1 DESC"""
  
    
    #sqlAlcaMergeJobsByAlcaIds=""" SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset,
  #pds.name AS skim
#FROM
  #wmbs_job j
#INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
#INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
#INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
#INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
#INNER JOIN processed_dataset pds          ON (dp.processed_dataset = pds.id)
#WHERE
  #ja.job IN
  #(SELECT
    #job
  #FROM
    #wmbs_job_assoc
  #WHERE
    #fileid IN (%s)
  #)
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name,
  #pds.name
#ORDER BY
  #1 DESC"""
    #sqlExprFilesByExprIds="""SELECT DISTINCT
  #f.id,
  #f.filesize,
  #f.events,
  #f.lfn
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_fileset_files fsf ON (f.id  = fsf.fileid)
#INNER JOIN wmbs_fileset fs        ON (fs.id = fsf.fileset)
#WHERE
  #fs.name = 'ExpressMergeable'
#AND f.id IN (%s)
#ORDER BY
  #1 DESC"""
    #sqlExprMergeJobsByExprIds="""SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles
#FROM
  #wmbs_job j
#INNER JOIN wmbs_job_assoc ja       ON (j.id = ja.job)
#INNER JOIN wmbs_file_details f     ON (f.id = ja.fileid)
#INNER JOIN wmbs_file_runlumi_map m ON (f.id = m.fileid)
#WHERE
  #ja.fileid IN (%s)
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time
#ORDER BY
  #1 DESC"""
    #sqlExprMergeFilesByExprIds="""SELECT DISTINCT
  #f.id,
  #f.filesize,
  #f.events,
  #f.lfn
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_fileset_files fsf ON (f.id  = fsf.fileid)
#INNER JOIN wmbs_fileset fs        ON (fs.id = fsf.fileset)
#INNER JOIN wmbs_file_parent fp    ON (f.id  = fp.child)
#WHERE
  #fs.name      = 'ExpressDBSUploadable'
#AND fp.parent IN (%s)
#ORDER BY
  #1 DESC"""
    #sqlAlcaMergeFilesByAlcaIds=""" SELECT DISTINCT
  #f.id,
  #f.filesize,
  #f.events,
  #pd.name as dataset,
  #f.lfn,
  #pds.name AS skim
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN dataset_path dp                ON (a.dataset_path_id    = dp.id)
#INNER JOIN primary_dataset pd             ON (dp.primary_dataset   = pd.id)
#INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
#INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
#INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
#INNER JOIN wmbs_file_parent fp            ON (f.id                 = fp.child)
#INNER JOIN processed_dataset pds          ON (dp.processed_dataset = pds.id)
#WHERE
  #dt.name      = 'ALCARECO'
#AND fs.name    = 'DBSUploadable'
#AND fp.parent IN (%s)
#ORDER BY
  #1 DESC"""
    #sqlAlcaFilesByAlcaIds=""" SELECT DISTINCT
  #f.id,
  #f.filesize,
  #f.events,
  #pd.name as dataset,
  #f.lfn,
  #pds.name AS skim
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN  dataset_path dp                ON (a.dataset_path_id    = dp.id)
#INNER JOIN  primary_dataset pd             ON (dp.primary_dataset   = pd.id)
#INNER JOIN  data_tier dt                   ON (dp.data_tier         = dt.id)
#INNER JOIN  wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
#INNER JOIN  wmbs_fileset fs                ON (fs.id                = fsf.fileset)
#INNER JOIN  processed_dataset pds          ON (dp.processed_dataset = pds.id)
#WHERE
  #dt.name   = 'ALCARECO'
#AND fs.name = 'Mergeable'
#AND f.id   IN (%s)
#ORDER BY
  #1 DESC"""
    ##sqlAlcaFilesByAlcaIds="""SELECT
  ##j.id as job_id,
  ##m.run as run_id,
  ##j.state_time as def_time,
  ##COUNT(DISTINCT ja.fileid) as inputfiles,
  ##pd.name as dataset,
  ##proc.name AS skim
##FROM
  ##wmbs_job j
##INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
##INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
##INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
##INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
##INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
##INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
##INNER JOIN processed_dataset PROC         ON (dp.processed_dataset = proc.id)
##WHERE
  ##ja.job IN
  ##(SELECT
    ##job
  ##FROM
    ##wmbs_job_assoc
  ##INNER JOIN wmbs_file_parent ON (parent = fileid)
  ##WHERE
    ##child IN (%s)
  ##)
##GROUP BY
  ##j.id,
  ##m.run,
  ##j.state_time,
  ##pd.name,
  ##proc.name
##ORDER BY
  ##1 DESC"""
  
    #sqlAlcaJobsByAlcaIds=""" SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset,
  #proc.name AS skim
#FROM
  #wmbs_job j
#INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
#INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
#INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
#INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
#INNER JOIN processed_dataset PROC         ON (dp.processed_dataset = proc.id)
#WHERE
  #ja.job IN
  #(SELECT j.id FROM  wmbs_job j 
      #INNER JOIN wmbs_job_assoc ja  ON (j.id      = ja.job)
      #INNER JOIN  wmbs_jobgroup jg ON (jg.id = j.jobgroup )
      #INNER JOIN wmbs_file_parent fp ON (fp.parent = ja.fileid)
  #WHERE
    #fp.child IN (%s) and jg.subscription !=  ( SELECT id FROM wmbs_subscription WHERE fileset=( SELECT id FROM wmbs_fileset WHERE name='Mergeable' ) )
  #)
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name,
  #proc.name
#ORDER BY
  #1 DESC"""
  
  ###########  and jg.subscription !=  ( SELECT id FROM wmbs_subscription WHERE fileset=( SELECT id FROM wmbs_fileset WHERE name='Mergeable' ) )
  ########################################################### queries To-Ids
  
    #sqlStrmIdsByRepk="""SELECT
  #streamer.streamer_id  str_id
#FROM
  #streamer
#JOIN stream ON streamer.stream_id = stream.id
#JOIN repack_job_streamer_assoc ON streamer.streamer_id =repack_job_streamer_assoc.streamer_id
#JOIN repack_job_def ON repack_job_def.job_id = repack_job_streamer_assoc.job_id
#JOIN repack_streamer_assoc ON repack_streamer_assoc.streamer_id = repack_job_streamer_assoc.streamer_id
#WHERE repack_streamer_assoc.repacked_id IN (%s)     
#ORDER BY 1 DESC"""
   ## OPTIONAL === AND rownum < :maxIdNum
    #sqlStrmIdsByExpr="""SELECT DISTINCT  a.streamer_id
#FROM
 #repack_streamer_assoc a
#INNER JOIN  wmbs_file_details f   ON (a.repacked_id = f.id)
#WHERE
   #f.id  IN (%s)
#ORDER BY
  #1 DESC"""
  
#################################
    #sqlExprIdsByStrm="""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN repack_streamer_assoc a ON (a.repacked_id = f.id)
#WHERE
  #a.streamer_id IN (%s)
#ORDER BY
  #1 DESC"""
  
    #sqlExprIdsByExprJob="""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN repack_streamer_assoc a ON (a.repacked_id = f.id)
#INNER JOIN streamer s ON (a.streamer_id = s.streamer_id)
  #WHERE
      #s.streamer_id IN
      #(SELECT DISTINCT
        #(express_job_streamer_assoc.streamer_id) AS distinct_1
      #FROM
        #express_job_streamer_assoc
      #WHERE
        #express_job_streamer_assoc.job_id IN (%s)
      #)"""
 
    #sqlExprIdsByExprMergeJob="""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_job_assoc ja ON (ja.fileid = f.id)
#WHERE
  #ja.job IN (%s)
#ORDER BY
  #1 DESC"""    
    #sqlExprIdsByExprMergeFile="""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_file_parent fp    ON (f.id  = fp.parent)
#WHERE fp.child IN (%s)
#ORDER BY
  #1 DESC"""
  #################################
    #sqlRepkIdsByStrm=sqlExprIdsByStrm
    ##"""SELECT DISTINCT
  ##f.id
##FROM
   ##wmbs_file_details f
##INNER JOIN  repack_streamer_assoc         ON (repack_streamer_assoc.repacked_id      = f.id)
##WHERE repack_streamer_assoc.streamer_id IN (%s)
##order by 1 DESC """
    #sqlRepkIdsByRepkJob="""SELECT DISTINCT
 #repack_streamer_assoc.repacked_id                                
#FROM
#repack_streamer_assoc
#JOIN repack_job_streamer_assoc  ON repack_job_streamer_assoc.streamer_id = repack_streamer_assoc.streamer_id
#JOIN repack_job_def ON repack_job_def.job_id     = repack_job_streamer_assoc.job_id AND repack_job_def.job_id =%s
#ORDER BY 1 DESC"""

    #sqlRepkIdsByRepkMergeJob=sqlExprIdsByExprMergeJob
    #"""SELECT DISTINCT
  #ja.fileid
#FROM
   #wmbs_job_assoc ja
#INNER JOIN  wmbs_job j            ON (j.id      = ja.job  and ja.job IN (%s))
#ORDER BY 1 DESC """
    #sqlRepkIdsByRepkMergeFile= sqlExprIdsByExprMergeFile
    #"""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
#INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.parent AND fp.child IN (%s))
#ORDER BY
  #1 DESC """
  
      
    #sqlRepkIdsByReco="""
#SELECT DISTINCT fp.parent
#FROM wmbs_file_parent fp
#WHERE fp.child IN
   #(%s )
#ORDER BY 1 DESC """
    
    
###################################
    #sqlRecoIdsByRepk="""
    #SELECT DISTINCT fp.child
#FROM wmbs_file_parent fp
#WHERE fp.parent IN
   #(%s )
#ORDER BY 1 DESC"""
##SELECT DISTINCT
  ##f.id
##FROM
   ##wmbs_file_details f 
##INNER JOIN  wmbs_file_parent fp    ON (f.id      = fp.child)
##INNER JOIN wmbs_file_parent fp2    ON (fp.parent = fp2.child )
##WHERE fp2.parent IN (%s)
##ORDER BY
  ##1 DESC"""
  
  
    #sqlRecoIdsByRecoJob="""SELECT DISTINCT fp.child FROM 
#wmbs_file_parent fp
#inner join wmbs_job_assoc ja on (fp.parent =ja.fileid)
#inner join wmbs_job j on (j.id = ja.job)
#where j.id IN (%s) order by 1 DESC """

    #sqlRecoIdsByRecoMergeJob=sqlExprIdsByExprMergeJob
    #"""SELECT DISTINCT
  #f.id
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_job_assoc ja              ON (ja.fileid          = f.id)
#WHERE  ja.job   IN (%s)
#ORDER BY
  #1 DESC"""
  
    #sqlRecoIdsByRecoMergeFile=sqlExprIdsByExprMergeFile
    #"""SELECT DISTINCT
  #f.id
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_file_parent fp            ON (f.id               = fp.parent)
#WHERE fp.child      IN (%s) """
  
    #sqlRecoIdsByAlca="""SELECT DISTINCT
  #fp.parent
#FROM wmbs_file_parent fp
#INNER JOIN wmbs_file_details f            ON (f.id                 = fp.child)
#WHERE
   #f.id IN (%s)
#ORDER BY
  #1 DESC"""

###########################################
  
  
    #sqlAlcaIdsByAlcaJob=sqlRecoIdsByRecoJob
    #"""  SELECT DISTINCT
  #fp.child
#FROM
   #wmbs_job_assoc ja
#INNER JOIN  wmbs_job j            ON (j.id      = ja.job  and ja.job IN (%s))
#INNER JOIN  wmbs_file_parent fp   ON ( ja.fileid   = fp.parent)
#ORDER BY 1 DESC"""
    #sqlAlcaIdsByAlcaMergeJob=sqlExprIdsByExprMergeJob
    #"""SELECT DISTINCT
  #f.id
#FROM
   #wmbs_file_details f
#INNER JOIN  wmbs_job_assoc ja              ON (ja.fileid            = f.id)
#WHERE ja.job IN (%s)
#ORDER BY
  #1 DESC"""
  
    #sqlAlcaIdsByAlcaMergeFile=sqlExprIdsByExprMergeFile
    #"""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN wmbs_file_parent fp            ON (f.id                 = fp.parent)
#WHERE fp.child IN (%s)
#ORDER BY
  #1 DESC"""


  ######################################################################################################## Excaptions!!!!!!!!!
  
  ############### from JOB to PREVIOUS IDs 
    #sqlStrmIdsByExpressJob="""SELECT
  #streamer.streamer_id
#FROM
  #streamer
#JOIN stream ON streamer.stream_id = stream.id
#WHERE
  #streamer.streamer_id IN
  #(SELECT DISTINCT
    #(express_job_streamer_assoc.streamer_id) AS distinct_1
  #FROM
    #express_job_streamer_assoc
  #WHERE
    #express_job_streamer_assoc.job_id IN (%s)
  #)
#ORDER BY
  #streamer.streamer_id DESC"""
  
  
    #sqlStrmIdsByRepkJob="""SELECT
  #streamer.streamer_id
#FROM
  #streamer
#JOIN stream ON streamer.stream_id = stream.id
#WHERE
  #streamer.streamer_id IN
  #(SELECT DISTINCT
    #(repack_job_streamer_assoc.streamer_id) AS distinct_1
  #FROM
    #repack_job_streamer_assoc
  #WHERE
    #repack_job_streamer_assoc.job_id IN (%s)
  #)
#ORDER BY
  #streamer.streamer_id DESC """
  
    #sqlRepkIdsbyRecoJob="""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
#INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
#INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
#INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
#INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
#INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
#INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.parent)
#WHERE
  #dt.name     = 'RAW'
#AND fs.name   = 'Mergeable'
#AND fp.child IN
  #( SELECT DISTINCT
    #f.id
  #FROM
    #wmbs_file_details f
  #INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
  #INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
  #INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
  #INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
  #INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
  #INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
  #INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
  #WHERE
    #dt.name   = 'RAW'
  #AND fs.name = 'DBSUploadable'
  #AND ja.job IN (%s)
  #)
#ORDER BY
  #1 DESC """
  
  
    #sqlRecoIdsByAlcaJob="""SELECT DISTINCT
  #f.id
#FROM
  #wmbs_file_details f
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
#INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
#INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
#INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
#INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
#INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
#INNER JOIN wmbs_job_assoc ja              ON (ja.fileid          = f.id)
#WHERE (dt.name                                                               ='ALCARECO'
#AND fs.name                                                                  = 'AlcaSkimmable')
#AND ja.job                                                                  IN (%s)
#ORDER BY
  #1 DESC"""
  
  #############################################################
  
    #sqlRepkJobsByItsIds="""SELECT
  #repack_job_def.job_id                                  AS job_id,
  #streamer.run_id                                        AS run_id,
  #repack_job_def.definition_time                         AS def_time,
  #COUNT(DISTINCT(repack_job_streamer_assoc.streamer_id)) AS num_streamers,
  #stream.name                                            AS stream,
  #job_status.status                                      AS status
#FROM
  #repack_job_def
#JOIN job_status                ON repack_job_def.job_status = job_status.id
#JOIN repack_job_streamer_assoc ON repack_job_def.job_id     = repack_job_streamer_assoc.job_id
#JOIN streamer                  ON streamer.streamer_id      = repack_job_streamer_assoc.streamer_id
#JOIN stream                    ON streamer.stream_id        = stream.id
#WHERE
  #repack_job_def.job_id IN (%s)
#GROUP BY
  #repack_job_def.job_id,
  #repack_job_def.definition_time,
  #streamer.run_id,
  #stream.name,
  #job_status.status
#ORDER BY
  #repack_job_def.job_id DESC """
  
    #sqlRecoJobsByItsIds="""SELECT 
  #j.id as job_id, 
  #m.run as run_id, 
  #j.state_time as def_time, 
  #COUNT(DISTINCT ja.fileid) as inputfiles, 
  #pd.name as dataset 
#FROM
  #wmbs_job j 
#INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job) 
#INNER JOIN wmbs_file_details f            ON (ja.fileid = f.id) 
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id = f.id) 
#INNER JOIN dataset_path dp                ON (dp.id     = a.dataset_path_id) 
#INNER JOIN primary_dataset pd             ON (pd.id     = dp.primary_dataset) 
#INNER JOIN wmbs_file_runlumi_map m        ON (f.id      = m.fileid) 
#WHERE 
  #j.id IN (%s)
#GROUP BY 
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name 
#ORDER BY 
  #1 DESC"""
  
    #sqlAlcaJobsByItsIds="""SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset,
  #proc.name AS skim
#FROM
  #wmbs_job j
#INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
#INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
#INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
#INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
#INNER JOIN processed_dataset PROC         ON (dp.processed_dataset = proc.id)
#WHERE
  #j.id IN (%s)
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name,
  #proc.name
#ORDER BY
  #1 DESC """
  
  
    #sqlExprJobsByItsIds="""SELECT
  #express_job_def.job_id                                  AS job_id,
  #streamer.run_id                                         AS run_id,
  #express_job_def.definition_time                         AS def_time,
  #COUNT(DISTINCT(express_job_streamer_assoc.streamer_id)) AS num_streamers
#FROM
  #express_job_def
#JOIN express_job_streamer_assoc ON express_job_def.job_id = express_job_streamer_assoc.job_id
#JOIN streamer                   ON streamer.streamer_id   = express_job_streamer_assoc.streamer_id
#WHERE
  #express_job_def.job_id IN (%s)
#GROUP BY
  #express_job_def.job_id,
  #express_job_def.definition_time,
  #streamer.run_id
#ORDER BY
  #express_job_def.job_id DESC """
  
##########################################################################
    #sqlExprJobByStrmIds="""SELECT
  #express_job_def.job_id                                  AS job_id,
  #streamer.run_id                                         AS run_id,
  #express_job_def.definition_time                         AS def_time,
  #COUNT(DISTINCT(express_job_streamer_assoc.streamer_id)) AS num_streamers
#FROM
  #express_job_def
#JOIN express_job_streamer_assoc ON express_job_def.job_id = express_job_streamer_assoc.job_id
#JOIN streamer                   ON streamer.streamer_id   = express_job_streamer_assoc.streamer_id
#WHERE
  #express_job_def.job_id IN
  #(SELECT DISTINCT
    #(express_job_streamer_assoc.job_id) AS distinct_1
  #FROM
    #express_job_streamer_assoc
  #WHERE
    #express_job_streamer_assoc.streamer_id IN (%s)
  #)
#GROUP BY
  #express_job_def.job_id,
  #express_job_def.definition_time,
  #streamer.run_id
#ORDER BY
  #express_job_def.job_id DESC """
  
    #sqlRepkJobByStrmIds="""SELECT
  #repack_job_def.job_id                                  AS job_id,
  #streamer.run_id                                        AS run_id,
  #repack_job_def.definition_time                         AS def_time,
  #COUNT(DISTINCT(repack_job_streamer_assoc.streamer_id)) AS num_streamers,
  #stream.name                                            AS stream,
  #job_status.status                                      AS status
#FROM
  #repack_job_def
#JOIN job_status                ON repack_job_def.job_status = job_status.id
#JOIN repack_job_streamer_assoc ON repack_job_def.job_id     = repack_job_streamer_assoc.job_id
#JOIN streamer                  ON streamer.streamer_id      = repack_job_streamer_assoc.streamer_id
#JOIN stream                    ON streamer.stream_id        = stream.id
#WHERE
  #repack_job_def.job_id IN
  #(SELECT DISTINCT
    #(repack_job_streamer_assoc.job_id) AS distinct_1
  #FROM
    #repack_job_streamer_assoc
  #WHERE
    #repack_job_streamer_assoc.streamer_id IN (%s)
  #)
#GROUP BY
  #repack_job_def.job_id,
  #repack_job_def.definition_time,
  #streamer.run_id,
  #stream.name,
  #job_status.status
#ORDER BY
  #repack_job_def.job_id DESC """

    #sqlRecoJobByRepkIds="""SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset 
#FROM 
  #wmbs_job j 
#INNER JOIN wmbs_job_assoc ja              ON (j.id      = ja.job)  
#INNER JOIN wmbs_file_details f            ON (ja.fileid = f.id) 
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id = f.id) 
#INNER JOIN dataset_path dp                ON (dp.id     = a.dataset_path_id) 
#INNER JOIN primary_dataset pd             ON (pd.id     = dp.primary_dataset) 
#INNER JOIN wmbs_file_runlumi_map m        ON (f.id      = m.fileid) 
#WHERE 
  #ja.job IN 
  #(SELECT
    #job
  #FROM
    #wmbs_job_assoc
  #WHERE
    #fileid IN
    #( SELECT DISTINCT
      #f.id
    #FROM
      #wmbs_file_details f
    #INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
    #INNER JOIN dataset_path dp                ON (a.dataset_path_id  = dp.id)
    #INNER JOIN primary_dataset pd             ON (dp.primary_dataset = pd.id)
    #INNER JOIN data_tier dt                   ON (dp.data_tier       = dt.id)
    #INNER JOIN wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
    #INNER JOIN wmbs_fileset fs                ON (fs.id              = fsf.fileset)
    #INNER JOIN wmbs_file_parent fp            ON (f.id               = fp.child)
    #WHERE
      #dt.name      = 'RAW'
    #AND fs.name    = 'DBSUploadable'
    #AND fp.parent IN (%s)
    #)
  #)
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name
#ORDER BY
  #1 DESC """
  
    #sqlAlcaJobByRecoIds=""" SELECT
  #j.id as job_id,
  #m.run as run_id,
  #j.state_time as def_time,
  #COUNT(DISTINCT ja.fileid) as inputfiles,
  #pd.name as dataset,
  #proc.name AS skim
#FROM
  #wmbs_job j
#INNER JOIN wmbs_job_assoc ja              ON (j.id                 = ja.job)
#INNER JOIN wmbs_file_details f            ON (ja.fileid            = f.id)
#INNER JOIN wmbs_file_dataset_path_assoc a ON (a.file_id            = f.id)
#INNER JOIN dataset_path dp                ON (dp.id                = a.dataset_path_id)
#INNER JOIN primary_dataset pd             ON (pd.id                = dp.primary_dataset)
#INNER JOIN wmbs_file_runlumi_map m        ON (f.id                 = m.fileid)
#INNER JOIN data_tier dt                   ON (dp.data_tier         = dt.id)
#INNER JOIN wmbs_fileset_files fsf         ON (f.id                 = fsf.fileid)
#INNER JOIN wmbs_fileset fs                ON (fs.id                = fsf.fileset)
#INNER JOIN processed_dataset PROC         ON (dp.processed_dataset = proc.id)
#WHERE
  #ja.job IN
  #( SELECT job FROM wmbs_job_assoc WHERE fileid IN (%s)
  #)
#AND dt.name ='ALCARECO'
#AND fs.name = 'AlcaSkimmable'
#GROUP BY
  #j.id,
  #m.run,
  #j.state_time,
  #pd.name,
  #proc.name
#ORDER BY
  #1 DESC"""
    
  ######################################################################### OLD NOT USED  
    ##sqlRepkIdsByRecoJobs="""SELECT DISTINCT fp.parent
##FROM wmbs_file_parent fp
##WHERE fp.child IN
   ##(SELECT fp.child
   ##FROM wmbs_file_parent fp
   ##INNER JOIN wmbs_job_assoc ja ON (fp.parent =ja.fileid)
   ##WHERE ja.fileid IN (SELECT DISTINCT
  ##f.id
##FROM
   ##wmbs_file_details f

##INNER JOIN  wmbs_job_assoc ja              ON (ja.fileid          = f.id)
##WHERE
 ##ja.job IN (%s)))
##ORDER BY
  ##1 DESC
##"""
   ###INNER JOIN wmbs_job j   ON (j.id    = ja.job)


###INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
###INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
###INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
###INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
###INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
###INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
  ###dt.name   = 'RAW'
###AND fs.name = 'DBSUploadable'
###AND
    ##sqlRecoJobsByItsIds="""SELECT j.id as job_id, m.run as run_id, j.state_time as def_time , COUNT(DISTINCT ja.fileid) as inputfiles, pd.name as dataset
            ##FROM  wmbs_job j INNER JOIN  wmbs_job_assoc ja ON (j.id = ja.job)
           ##INNER JOIN  wmbs_file_details f ON (ja.fileid = f.id)
           ##INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
           ##INNER JOIN  dataset_path dp ON (dp.id = a.dataset_path_id)
           ##INNER JOIN  primary_dataset pd ON (pd.id = dp.primary_dataset)
           ##INNER JOIN  wmbs_file_runlumi_map m ON (f.id = m.fileid)
            ##WHERE j.id IN (%s)
            ##GROUP BY j.id, m.run, j.state_time, pd.name
            ##ORDER BY  1 DESC """
            
    ##sqlRecoJobsByRepakIds= """SELECT
  ##j.id as job_id,
  ##m.run as run_id,
  ##j.state_time as def_time,
  ##COUNT(DISTINCT ja.fileid) as inputfiles,
  ##pd.name as dataset
##FROM
   ##wmbs_job j
##INNER JOIN  wmbs_job_assoc ja              ON (j.id      = ja.job)
##INNER JOIN  wmbs_file_details f            ON (ja.fileid = f.id)
##INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id = f.id)
##INNER JOIN  dataset_path dp                ON (dp.id     = a.dataset_path_id)
##INNER JOIN  primary_dataset pd             ON (pd.id     = dp.primary_dataset)
##INNER JOIN  wmbs_file_runlumi_map m        ON (f.id      = m.fileid)
##WHERE
  ##ja.job IN
  ##(SELECT
    ##job
  ##FROM
     ##wmbs_job_assoc
  ##WHERE
    ##fileid IN
    ##(SELECT DISTINCT
      ##f.id
    ##FROM
       ##wmbs_file_details f
    ##INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
    ##INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
    ##INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
    ##INNER JOIN  data_tier dt                   ON (dp.data_tier       = dt.id)
    ##INNER JOIN  wmbs_fileset_files fsf         ON (f.id               = fsf.fileid)
    ##INNER JOIN  wmbs_fileset fs                ON (fs.id              = fsf.fileset)
    ##INNER JOIN  wmbs_file_parent fp            ON (f.id               = fp.child)
    ##WHERE
      ##dt.name      = 'RAW'
    ##AND fs.name    = 'DBSUploadable'
    ##AND fp.parent IN (%s)
    ##)
  ##)
##GROUP BY
  ##j.id,
  ##m.run,
  ##j.state_time,
  ##pd.name
##ORDER BY
  ##1 DESC """
  
    ##sqlStrmByRepakJob="""  SELECT
  ##streamer.streamer_id AS ids
##FROM
  ##streamer
##WHERE
  ##streamer.streamer_id IN
  ##(SELECT DISTINCT
    ##(repack_job_streamer_assoc.streamer_id) AS distinct_1
  ##FROM
    ##repack_job_streamer_assoc
  ##WHERE
    ##repack_job_streamer_assoc.job_id IN (%s)
  ##)
##ORDER BY
  ##streamer.streamer_id DESC"""
  
    ##sqlRepkJobByRepakJob="""
  ##SELECT
  ##repack_job_def.job_id                                  AS job_id,
  ##streamer.run_id                                        AS run_id,
  ##repack_job_def.definition_time                         AS def_time,
  ##COUNT(DISTINCT(repack_job_streamer_assoc.streamer_id)) AS num_streamers,
  ##stream.name                                            AS stream,
  ##job_status.status                                      AS status
##FROM
  ##repack_job_def
##JOIN job_status                ON repack_job_def.job_status = job_status.id
##JOIN repack_job_streamer_assoc ON repack_job_def.job_id     = repack_job_streamer_assoc.job_id
##JOIN streamer                  ON streamer.streamer_id      = repack_job_streamer_assoc.streamer_id
##JOIN stream                    ON streamer.stream_id        = stream.id
##WHERE
  ##repack_job_def.job_id IN (%s)
##GROUP BY
  ##repack_job_def.job_id,
  ##repack_job_def.definition_time,
  ##streamer.run_id,
  ##stream.name,
  ##job_status.status
##ORDER BY
  ##repack_job_def.job_id DESC"""
    ##sqlStrmByExprJob="""SELECT
    ##streamer.streamer_id AS ids
  ##FROM
    ##streamer
  ##WHERE
    ##streamer.streamer_id IN
    ##(SELECT DISTINCT
      ##(express_job_streamer_assoc.streamer_id) AS distinct_1
    ##FROM
      ##express_job_streamer_assoc
    ##WHERE
      ##express_job_streamer_assoc.job_id IN (%s)
    ##)
  ##ORDER BY
    ##streamer.streamer_id DESC """
    ##sqlExprJobByExprJob="""SELECT
  ##express_job_def.job_id                                  AS job_id,
  ##streamer.run_id                                         AS run_id,
  ##express_job_def.definition_time                         AS def_time,
  ##COUNT(DISTINCT(express_job_streamer_assoc.streamer_id)) AS num_streamers
##FROM
  ##express_job_def
##JOIN express_job_streamer_assoc ON express_job_def.job_id = express_job_streamer_assoc.job_id
##JOIN streamer                   ON streamer.streamer_id   = express_job_streamer_assoc.streamer_id
##WHERE
  ##express_job_def.job_id IN (%s)
##GROUP BY
  ##express_job_def.job_id,
  ##express_job_def.definition_time,
  ##streamer.run_id
##ORDER BY
  ##express_job_def.job_id DESC"""