#!/usr/bin/env python
"""
_Short information about the last files_

T0Mon: this class is used for all the tier plots: express/raw/reco/alcareco plots
"""


from WMCore.Database.DBFormatter import DBFormatter
#
class LastFilesSummary(DBFormatter): 

   sql="""SELECT DISTINCT f.id,f.filesize as filesize,j.state_time as time, f.events as events,
    pd.name as dataset, CASE WHEN  s.name='A' or s.name='Express' THEN s.name 
    ELSE 'others' END as stream
    FROM
    wmbs_file_details f
    INNER JOIN  wmbs_file_dataset_path_assoc a ON (a.file_id          = f.id)
    INNER JOIN  dataset_path dp                ON (a.dataset_path_id  = dp.id)
    INNER JOIN  primary_dataset pd             ON (dp.primary_dataset = pd.id)
    INNER JOIN  wmbs_job_assoc  ja             ON (ja.fileid =f.id)
    INNER JOIN wmbs_job          j              ON (j.id= ja.job)
    INNER JOIN dataset_run_stream_assoc dr ON (dr.primary_dataset_id= pd.id)
    INNER JOIN stream s ON (s.id= dr.stream_id)
  WHERE j.state_time> :stime AND j.state_time< :etime %s 
  ORDER BY j.state_time DESC"""
   tier_option="""AND dp.data_tier =      (SELECT   data_tier.id FROM data_tier WHERE data_tier.name = :tier  )"""
   
   expressSql="""SELECT  DISTINCT jd.job_id, st.filesize as filesize,st.insert_time as time, st.events as events,
    pd.name as dataset, CASE WHEN  s.name='A' or s.name='Express' THEN s.name 
    ELSE 'others' END as stream
    FROM
    express_job_def jd
    INNER JOIN express_job_streamer_assoc esa ON (esa.job_id = jd.job_id)
    INNER JOIN streamer st ON (st.streamer_id = esa.streamer_id)
    INNER JOIN stream s ON (st.stream_id = s.id)
    INNER JOIN stream_special_dataset_assoc sd ON (sd.stream_id = s.id)
    INNER JOIN primary_dataset pd ON (pd.id= sd.primary_dataset_id)
   WHERE st.insert_time> :stime AND st.insert_time< :etime 
    ORDER BY st.insert_time DESC """
  
   def execute(self, stime,etime,tier, conn = None, transaction = False):
        bindVars={}
        try:
          bindVars = {"stime": str(stime),"etime": str(etime)}
        except Exception:
            return {"Error" : "wrong time"}
        try:
            if len(str(tier))>0:
             bindVars.update({"tier": str(tier)})
            else: self.tier_option=""
        except:
            pass
        self.sql=self.sql%(self.tier_option)
        
        if tier.lower()=='express':
           self.sql=self.expressSql
           bindVars = {"stime": str(stime),"etime": str(etime)}
        #print self.sql
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result) 
 
