#!/usr/bin/env python
"""
_Information on express jobs by status_

T0Mon: this class is used in the detail pages 
"""


from T0.DAS.Database.Oracle.LongResultQuery import LongResultQuery 
statuses=["New","Used","Success","Failure","Abandoned"]
class ExpressJobsByStatus(LongResultQuery):
   """
   Repack and Express jobs are different from all the others: their statuses are: "New","Used","Success","Failure","Abandoned".
   This class serves Express jobs, with changes only to the sql variables the subclass RepackJobByStatus serves the Repack jobs.   
   """
   # TO DO: fetch statuses from the DB!
   sql="""
        SELECT
           express_job_def.job_id                                  AS  job_id,
           streamer.run_id                                         AS run_id,
           express_job_def.definition_time                         AS def_time,
          COUNT(DISTINCT( express_job_streamer_assoc.streamer_id)) AS num_streamers
        FROM
           express_job_def
        JOIN  express_job_streamer_assoc ON  express_job_def.job_id =  express_job_streamer_assoc.job_id
        JOIN  streamer                   ON  streamer.streamer_id   =  express_job_streamer_assoc.streamer_id
        WHERE
           express_job_def.job_status = (SELECT job_status.id FROM job_status WHERE job_status.status= :status)
           %s
        GROUP BY
           express_job_def.job_id,
           express_job_def.definition_time,
           streamer.run_id
        ORDER BY
           express_job_def.job_id DESC
    """
  
   sqlWhereRun=" AND streamer.run_id= :run"
   
   def execute(self, status,run,uplimit,downlimit, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL 
        """
        #print self.sql
        global statuses
        if status not in statuses:
          return {"Error" : "wrong  status", "hint": statuses} 
        if len(run)>0:
          try:
            runid=int(run)
            self.sql=self.sql %(self.sqlWhereRun)
            bindVars = {"status": status,"run":run,"uplimit":uplimit,"downlimit":downlimit}
          except Exception:
            pass
        if 'bindVars' not in locals():
           bindVars = {"status": status,"uplimit":uplimit,"downlimit":downlimit}
           self.sql=self.sql %("")
        
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result)
         
#################################################################################################
from WMCore.Database.DBFormatter import DBFormatter
class ExpressJobsByStatus_count(DBFormatter):

  sql="""SELECT COUNT( DISTINCT express_job_def.job_id)
  FROM  express_job_def %s
  WHERE express_job_def.job_status = (SELECT job_status.id FROM job_status WHERE job_status.status= :status)"""
  sqlJoinRun="""
   INNER JOIN  express_job_streamer_assoc ON  (express_job_def.job_id =  express_job_streamer_assoc.job_id)
   INNER JOIN  streamer                   ON  (streamer.streamer_id   =  express_job_streamer_assoc.streamer_id AND  streamer.run_id= :run) 
   """
  def execute(self, status,run, conn = None, transaction = False):
        global statuses
        if status not in statuses:
          return {"Error" : "wrong  status", "hint": statuses} 
        if len(run)>0:
          try:
            runid=int(run)
            self.sql=self.sql %(self.sqlJoinRun)
            bindVars = {"status": status,"run":run}
          except Exception:
            pass
        if 'bindVars' not in locals():
           bindVars = {"status": status}
           self.sql=self.sql %("")
        
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        return self.format(result)