#!/usr/bin/env python
"""
_Information on repack merge jobs by status_

T0Mon: this class is used in the detail pages 
"""

from T0.DAS.Database.Oracle.LongResultQuery import LongResultQuery 

statuses=["acquired","complete","failed"]
sqlWhereAcquired=""" AND wmbs_job.state!= ( SELECT id FROM wmbs_job_state WHERE name = 'cleanout'    ) """
    
sqlWhereFailed=""" AND wmbs_job.state=(SELECT id FROM wmbs_job_state WHERE name = 'cleanout' ) 
                       AND wmbs_job.outcome=0 """
sqlWhereComplete=""" AND wmbs_job.state=(SELECT id FROM wmbs_job_state WHERE name = 'cleanout' ) 
                         AND wmbs_job.outcome=1 """
                         
                         
sqlRun="""    INNER JOIN  wmbs_file_runlumi_map m  ON (wmbs_job_assoc.fileid = m.fileid and  m.run = :run)  """

    
        
class RepackMergeJobsByStatus(LongResultQuery):
   """
   This class uses a combination of global vars and local ones to build dinamically the sql code: all the other *ByStatus  classes (with the exception of express/repack) are inheriting from this one. 
   """
   sqlFileSet="'Mergeable'"
   dataTier="'RAW'"
   sql="""SELECT
       wmbs_job.id                            AS job_id,
       wmbs_file_runlumi_map.run              AS run_id,
       wmbs_job.state_time                    AS state_time,
      COUNT(DISTINCT( wmbs_job_assoc.fileid)) AS inputfiles,
       primary_dataset.name                   AS dataset
    FROM
       wmbs_job
    JOIN  wmbs_jobgroup                ON  wmbs_job.jobgroup                            =  wmbs_jobgroup.id
    JOIN  wmbs_job_assoc               ON  wmbs_job.id                                  =  wmbs_job_assoc.job
    JOIN  wmbs_file_details            ON  wmbs_file_details.id                         =  wmbs_job_assoc.fileid
    JOIN  wmbs_file_runlumi_map        ON  wmbs_file_details.id                         =  wmbs_file_runlumi_map.fileid
    JOIN  wmbs_file_dataset_path_assoc ON  wmbs_file_details.id                         =  wmbs_file_dataset_path_assoc.file_id
    JOIN  dataset_path                 ON  wmbs_file_dataset_path_assoc.dataset_path_id =  dataset_path.id
    JOIN  primary_dataset              ON  primary_dataset.id                           =  dataset_path.primary_dataset
    %s
    WHERE
       dataset_path.data_tier =
      (SELECT data_tier.id  FROM data_tier
      WHERE  data_tier.name = %s)
    AND  wmbs_jobgroup.subscription =
      (SELECT  wmbs_subscription.id  FROM   wmbs_subscription
      WHERE  wmbs_subscription.fileset =
        (SELECT  wmbs_fileset.id FROM   wmbs_fileset
        WHERE wmbs_fileset.name = %s))
    %s
    GROUP BY
       wmbs_job.id,
       wmbs_job.state_time,
       wmbs_file_runlumi_map.run,
       primary_dataset.name
    ORDER BY
       wmbs_job.id DESC  """
  

    
   def buildQuery(self):
        self.sql=self.sql % (self.sqlRun,self.dataTier,self.sqlFileSet,self.whereCond)
        
        
   def execute(self, run,status,uplimit,downlimit, conn = None, transaction = False):
        """
        _execute_
        Execute the SQL for the given  statistics and then format and return
        the result.
        """
        bindVars={}
        global statuses,sqlWhereAcquired,sqlWhereFailed,sqlWhereComplete,sqlRun
        status=status.lower()
        if (len(status)<1) or (status not in statuses): 
          return {"Error" : "wrong status name", "hint": statuses}

        if status==statuses[0]:
          self.whereCond=sqlWhereAcquired
        elif status==statuses[1]:
           self.whereCond=sqlWhereComplete
        elif status==statuses[2]:
          self.whereCond=sqlWhereFailed
        try:
          if int(run)>0:
           self.sqlRun=sqlRun
           self.buildQuery()
           #self.sql=self.sql % (self.sqlRun,self.dataTier,self.sqlFileSet,self.whereCond)
           bindVars = {"run": run,"uplimit":uplimit,"downlimit":downlimit}
          else: 
            raise Exception('zeroRun')
        except Exception:
            bindVars={"uplimit":uplimit,"downlimit":downlimit}
            self.sqlRun=""
            self.buildQuery()
            #self.sql=self.sql % ("",self.dataTier,self.sqlFileSet,self.whereCond)
        
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        
        return self.formatDict(result) 
  
    
#################################################################################################
from WMCore.Database.DBFormatter import DBFormatter

class RepackMergeJobsByStatus_count(DBFormatter):
   sqlFileSet="'Mergeable'"
   dataTier="'RAW'"
   sql="""SELECT COUNT (DISTINCT   wmbs_job.id)
    FROM
       wmbs_job
    JOIN  wmbs_jobgroup                ON  wmbs_job.jobgroup                            =  wmbs_jobgroup.id
    JOIN  wmbs_job_assoc               ON  wmbs_job.id                                  =  wmbs_job_assoc.job
    JOIN  wmbs_file_details            ON  wmbs_file_details.id                         =  wmbs_job_assoc.fileid
    JOIN  wmbs_file_dataset_path_assoc ON  wmbs_file_details.id                         =  wmbs_file_dataset_path_assoc.file_id
    JOIN  dataset_path                 ON  wmbs_file_dataset_path_assoc.dataset_path_id =  dataset_path.id
    %s
    WHERE
       dataset_path.data_tier =
      (SELECT data_tier.id  FROM data_tier
      WHERE  data_tier.name = %s)
    AND  wmbs_jobgroup.subscription =
      (SELECT  wmbs_subscription.id  FROM   wmbs_subscription
      WHERE  wmbs_subscription.fileset =
        (SELECT  wmbs_fileset.id FROM   wmbs_fileset
        WHERE wmbs_fileset.name = %s))
 %s
    """ 
   def buildQuery(self):
        self.sql=self.sql % (self.sqlRun,self.dataTier,self.sqlFileSet,self.whereCond)
        
        
   def execute(self, run,status, conn = None, transaction = False):
        bindVars={}
        global statuses,sqlWhereAcquired,sqlWhereFailed,sqlWhereComplete,sqlRun
        status=status.lower()
        if (len(status)<1) or (status not in statuses): 
          return {"Error" : "wrong status name", "hint": statuses}

        if status==statuses[0]:
          self.whereCond=sqlWhereAcquired
        elif status==statuses[1]:
           self.whereCond=sqlWhereComplete
        elif status==statuses[2]:
          self.whereCond=sqlWhereFailed
        try:
          if int(run)>0:
           self.sqlRun=sqlRun
           self.buildQuery()
           #self.sql=self.sql % (self.sqlRun,self.dataTier,self.sqlFileSet,self.whereCond)
           bindVars = {"run": run}
          else: 
            raise Exception('zeroRun')
        except Exception:
            bindVars={}
            self.sqlRun=""
            self.buildQuery()
          
        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)
        return self.format(result) 
  