
from WMCore.HTTPFrontEnd.WorkQueue.Services.ServiceInterface import ServiceInterface
from WMCore.DAOFactory import DAOFactory

class Tier0RECOService(ServiceInterface):
    """
    _Tier0RECOService_
    
    Provice the tier0 REST services for RECO group 
    """
    
    def register(self):
        
        daoFactory = DAOFactory(package = "T0.DAS.Database",
                                     logger = self.model,
                                     dbinterface = self.model.dbi)
        
        # run related daos
        self.model.addDAO('GET', 'runs', 'Runs.Run', daoFactory=daoFactory)
        self.model.addDAO('GET', 'runsbystatus', 'Runs.RunsByStates', args=['run_status'], 
                          daoFactory=daoFactory)
        self.model.addDAO('GET', 'runsbyrecostatus', 'Runs.RunsByRecoStartStatus', args=['reco_status'], 
                          daoFactory=daoFactory)
        
        self.model.addDAO('GET', 'isexpresscomplete', 'Runs.IsRunExpressComplete', args=['run'], 
                          daoFactory=daoFactory)
        
        #Timeing related daos
        # arguments : run
        self.model.addDAO('GET', 'recostatusbyrun', 'Timing.RecoStartStatusByRun', args=['run'],
                          daoFactory=daoFactory)
        self.model.addDAO('GET', 'recotimeleft', 'Timing.RecoStartTimeLeft', 
                          daoFactory=daoFactory)
        # arguments : run
        self.model.addDAO('GET', 'recotimeleftbyrun', 'Timing.RecoStartTimeLeftByRun', args=['run'],
                          daoFactory=daoFactory)
        
        #Block info
        self.model.addDAO('GET', 'injectiontimebydataset', 'Blocks.BlockInfoByDatasetPath', 
                          args=['datasetpath'],
                          daoFactory=daoFactory)
        
        # Config related info
        # arguments : run, stream
        self.model.addDAO('GET', 'express_config',
                          'Configs.ExpressStreamConfig',
                          args=['run','stream'],
                          daoFactory=daoFactory)
        # arguments : run, primary dataset
        self.model.addDAO('GET', 'reco_config',
                          'Configs.RecoDatasetConfig',
                          args=['run','dataset'],
                          daoFactory=daoFactory)
        
