#!/usr/bin/env python
"""
_InsertJob_

Utils for inserting rows into the job_instance and job_def tables as well
as updating the information in them.
"""

__revision__ = "$Id: InsertPerformanceData.py,v 1.2 2009/10/13 19:33:07 sryu Exp $"
__version__ = "$Revision: 1.2 $"

import time
import logging

from T0.State.Database.Reader.ListJobs import getNextJobID


def insertPerfData(dbConn, jobID, tableName, perfData):
    
    sqlQuery = """ INSERT INTO timing  
                     (job_id, type_id, cpu_id, num_events, avg_event_cpu, max_event_cpu,
                      min_event_cpu, avg_event_time, max_event_time, 
                      min_event_time, total_event_cpu, total_job_cpu,
                      total_job_time)
                   VALUES 
                     (:job_id, 
                     (SELECT id FROM job_table_name WHERE name = :table_name), 
                     :cpu_id, :num_events,
                     :avg_event_cpu, :max_event_cpu, :min_event_cpu, 
                     :avg_event_time, :max_event_time, :min_event_time, 
                     :total_event_cpu, :total_job_cpu, :total_job_time)""" 
                                       
    bindVars = {}
    bindVars['job_id'] = jobID
    bindVars['table_name'] = tableName
    bindVars.update(perfData)
    
    dbConn.execute(sqlQuery, bindVars)

    return

def insertSystemCPUInfo(dbConn, cpuInfo):
    
    sqlQuery = """SELECT cpu_id FROM sys_cpu 
                    WHERE cpu_model = :cpu_model AND avg_core_speed = :avg_core_speed 
                    AND total_cores = :total_cores """
    
    bindVars = {}
    bindVars.update(cpuInfo)        
    dbConn.execute(sqlQuery, bindVars)
    results = dbConn.fetchall()
    
    if len(results) == 1:
        return results[0][0]
    
    sqlQuery = """ SELECT sys_cpu_SEQ.nextVal FROM dual"""
    
    dbConn.execute(sqlQuery)
    results = dbConn.fetchall()
    cpuID = results[0][0]
        
    sqlQuery =  """ INSERT INTO sys_cpu 
                        (cpu_id, cpu_model, avg_core_speed, total_cores) 
                    VALUES 
                        (:cpu_id, :cpu_model, :avg_core_speed, :total_cores)
                """
    bindVars['cpu_id']= cpuID
    dbConn.execute(sqlQuery, bindVars)

    return cpuID