#!/usr/bin/env python
"""
_AlcaSkim_

Job handler for AlcaSkim jobs.
"""
__revision__ = "$Id: AlcaSkim.py,v 1.12 2010/07/30 16:00:36 hufnagel Exp $"
__version__ = "$Revision: 1.12 $"

import logging

from WMCore.WMBS.Fileset import Fileset as WMBSFileset

from T0.Globals import retrieveAlcaSubscription
from T0.State.Database.Writer import InsertRunConfig
from T0.Tier0Accountant.WMBSBaseHandler import WMBSBaseHandler

class AlcaSkim(WMBSBaseHandler):
    """
    _AlcaSkim_

    Job handler for AlcaSkim jobs.
    """
    
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        Given a file that was returned by a alca skim job, insert it into
        T0AST(and WMBS) and setup all the necessary associations.        
        """
        
        logging.debug("AlcaSkimHandler handles successful job ID:%s"  
                      % jobInfo["JOB_IDENTIFIER"])
        
        wmbsJob = WMBSBaseHandler.handleSuccessfulJob(self, jobInfo, jobFiles)
        if wmbsJob == None:
            # error message handled by base class
            return

        # update the PSetHash 
        if len(jobFiles) > 0:

            t0astFile = jobFiles[0]
            InsertRunConfig.updatePSetHash(self.t0astDBConn,
                                           "alca_config",
                                           t0astFile["RUN_ID"],
                                           t0astFile["DATASET_ID"],
                                           t0astFile["PSET_HASH"])

        return
    
    
    def getFilesetListByFile(self, wmbsFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list depending on t0astFile for next processing
        The list will depends on various factor (tier, merge by size conditon)
        """
               
        filesetList = []   
        filesetList.append(WMBSFileset(name = "Mergeable"))
        
        return filesetList

