#!/usr/bin/env python
"""
_Express_

Job handler for Express jobs.
"""

__revision__ = "$"
__version__ = "$"

import logging

from WMCore.WMBS.Fileset import Fileset as WMBSFileset

from T0.Tier0Accountant.StreamerBaseHandler import StreamerBaseHandler
from T0.State.Database.Writer import InsertRepacked
from T0.State.Database.Writer import InsertDataset

class Express(StreamerBaseHandler):
    """
    _Express_

    Job handler for Express jobs.
    """
    
    def setAssociation(self, jobInfo, t0astFile):
        """
        _setAssocitation_
        This function should set association between input files and output file
        (t0astFile). if there are other association to set (i.e. dataset or tier
        to output file, etc should implemented in child class

        """
        InsertDataset.assocExpressFileStreamDataset(self.t0astDBConn,
                                                    t0astFile["ID"],
                                                    t0astFile["STREAM"],
                                                    t0astFile["DATA_TIER"],
                                                    t0astFile["PROCESSED_DATASET"],
                                                    t0astFile["PRIMARY_DATASET"])
        
        InsertRepacked.expressStreamerAssoc(self.t0astDBConn,
                                            jobInfo["JOB_IDENTIFIER"],
                                            t0astFile["ID"])

    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list depending on t0astFile for next processing
        The list will depends on various factor (tier, merge by size conditon)
        If the file can be added in the fileset one at a time without waiting for 
        the job group to be finished, use this list

        """
        filesetList = []
        filesetList.append(WMBSFileset(name = "ExpressMergeable"))
        return filesetList

