#!/usr/bin/env python
"""
_MergePack_

Job handler for MergePack jobs.
"""

__revision__ = "$"
__version__ = "$"

import logging
from sets import Set

from WMCore.WMBS.File import File as WMBSFile
from WMCore.WMBS.Fileset import Fileset as WMBSFileset

from T0.Tier0Accountant.WMBSBaseHandler import WMBSBaseHandler

class MergePack(WMBSBaseHandler):
    """
    _MergePack_

    Job handler for Merge jobs.
    """
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        Given a file that was returned by a merge pack job, insert it into T0AST and
        setup all the necessary associations.
        """
        # update merge flag
        for jobFile in jobFiles:
            jobFile["MERGED"] = True

        WMBSBaseHandler.handleSuccessfulJob(self, jobInfo, jobFiles)

    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list depending on t0astFile for next processing
        The list will depends on various factor (tier, merge by size conditon)
        If the file can be added in the fileset one at a time without waiting for 
        the job group to be finished, use this list
        """
        if t0astFile["DATA_TIER"] == "DQM":
            return [ WMBSFileset(name = "DQMenable") ]
        elif t0astFile["DATA_TIER"] == "ALCARECO":
            return [ WMBSFileset(name = "ExpressDBSUploadable"),
                     WMBSFileset(name = "PromptCalibrationable") ]
        elif t0astFile["DATA_TIER"] == "ALCAPROMPT":
            return [ WMBSFileset(name = "ExpressDBSUploadable"),
                     WMBSFileset(name = "AlcaHarvestable") ]
        else:
            return [ WMBSFileset(name = "ExpressDBSUploadable") ]

    def deleteInputFiles(self):
        """
        _deleteInputFiles_ 
        
        override base class method to cleanup input files immediately
        """
        return True
