#!/usr/bin/env python
"""
_PromptReco_

Job handler for PromptReco jobs.
"""

__revision__ = "$Id: PromptReco.py,v 1.30 2010/07/30 16:00:36 hufnagel Exp $"
__version__ = "$Revision: 1.30 $"

import logging

from WMCore.WMBS.Fileset import Fileset as WMBSFileset

from T0.Globals import retrieveRecoSubscription
from T0.State.Database.Writer import InsertRunConfig
from T0.Tier0Accountant.WMBSBaseHandler import WMBSBaseHandler

class PromptReco(WMBSBaseHandler):
    """
    _PromptReco_

    Job handler for PromptReco jobs.
    """
    
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        Given a file that was returned by a prompt reco job, insert it into
        T0AST(and WMBS) and setup all the necessary associations.        
        """
        
        logging.debug("PromptRecoHandler handles successful job ID:%s"  
                      % jobInfo["JOB_IDENTIFIER"])
        
        wmbsJob = WMBSBaseHandler.handleSuccessfulJob(self, jobInfo, jobFiles)
        if wmbsJob == None:
            # error message handled by base class
            return

        # update the PSetHash, only do this once
        if len(jobFiles) > 0:

            t0astFile = jobFiles[0]
            InsertRunConfig.updatePSetHash(self.t0astDBConn,
                                           "reco_config",
                                           t0astFile["RUN_ID"],
                                           t0astFile["DATASET_ID"],
                                           t0astFile["PSET_HASH"])

        return
    
    
    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list which apply to given file 
        This can should be used if the behavior differs by output file (t0astFile) type 
        i.e) if the input files are no longer needed since they are intermediate files, 
        they can be added to Deletable fileset  
        """
        if t0astFile["DATA_TIER"] == "ALCARECO":
            return [ WMBSFileset(name = "AlcaSkimmable") ]
        else:
            return [ WMBSFileset(name = "Mergeable") ]
