#!/usr/bin/env python
"""
_Repack_

Job handler for Repack jobs.
"""

__revision__ = "$Id: Repack.py,v 1.20 2010/02/15 14:28:57 hufnagel Exp $"
__version__ = "$Revision: 1.20 $"

import logging
from sets import Set

from WMCore.WMBS.Fileset import Fileset as WMBSFileset

from T0.Tier0Accountant.StreamerBaseHandler import StreamerBaseHandler
from T0.State.Database.Reader import ListDatasets
from T0.State.Database.Writer import InsertRepacked
from T0.State.Database.Writer import InsertDataset

class Repack(StreamerBaseHandler):
    """
    _Repack_

    Job handler for repack jobs.
    """
    
    def handleSuccessfulJob(self, jobInfo, jobFiles):
        """
        _handleSuccessfulJob_

        Given a list of files that were produced by a repack job, insert them
        into T0AST and setup all necessary associations.
        """
        newJobFiles = []
        for t0astFile in jobFiles:
            newJobFiles.append(t0astFile)
                
        StreamerBaseHandler.handleSuccessfulJob(self, jobInfo, newJobFiles)
        
        return
    
    
    def setAssociation(self, jobInfo, t0astFile):
        """
        _setAssocitation_
        This function should set association between input files and output file
        (t0astFile). if there are other association to set (i.e. dataset or tier
        to output file, etc should implemented in child class
        """
        
        # all the repack processed dataset name should be already in the datble
        InsertDataset.insertDatasetByName(self.t0astDBConn, 
                                          t0astFile["PRIMARY_DATASET"],
                                          t0astFile["PROCESSED_DATASET"],
                                          t0astFile["DATA_TIER"])
        
        InsertDataset.assocWMBSFileDatasetByNames(self.t0astDBConn,
                                    t0astFile["ID"],
                                    t0astFile["PRIMARY_DATASET"], 
                                    t0astFile["PROCESSED_DATASET"],
                                    t0astFile["DATA_TIER"])
        
        InsertRepacked.repackedStreamerAssoc(self.t0astDBConn,
                                             jobInfo["JOB_IDENTIFIER"],
                                             t0astFile["ID"])
        return
    
    
    def getFilesetListByFile(self, t0astFile):
        """
        _getFilesetListByFile_ 
        
        retrieve the fileset list depending on t0astFile for next processing
        The list will depends on various factor (tier, merge by size conditon)
        If the file can be added in the fileset one at a time without waiting for 
        the job group to be finished, use this list
        
        """
        
        filesetList = []

        if t0astFile["MERGED"]:
            filesetList.append(WMBSFileset(name = "DBSUploadable"))
            filesetList.append(WMBSFileset(name = "Reconstructable"))
        else:
            filesetList.append(WMBSFileset(name = "Mergeable"))
            
        return filesetList
